#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, glob, shutil
from PyQt4.QtGui import QMessageBox # Ajouté le 20/07/11
from gui_modules_animation.animation_base_encodageFiltre import Base_EncodageFiltre
from gui_modules_common.ffmpeg_gui import WidgetFFmpeg
from gui_modules_common.EkdWidgets import EkdSaveDialog
from moteur_modules_common.EkdConfig import EkdConfig
###############
from moteur_modules_common.EkdCompatiblePython2Et3 import EkdPrint
###############

class AnimationEncodageWeb(Base_EncodageFiltre):
	"""Cadre de Animation -> Transcodage -> Pour le web. But: encodages spécifiques à youtube, dailymotion..."""

	def __init__(self):

		# Identifiant de la classe
		self.idSection = "animation_encodage_web"
		# Pas d'"empilement" car pas de paramètre d'encodage
		self.stacked = None
		
		# ------------------------------------------------------------------- #
		# Chemin des répertoires temporaires pour la gestion des fichiers
		# mod (extension .mod). Ce sont (apparemment) des fichiers mpeg avec
		# une extension .mod. Les fichiers en question ont juste besoin
		# d'être renommés avec une extension .mpg avant le traitement.
		# ------------------------------------------------------------------- #
		self.repTempEntree = EkdConfig.getTempDir() + os.sep
		# création des répertoires temporaires
		if os.path.isdir(self.repTempEntree) is False: os.makedirs(self.repTempEntree)	
        	# Chemin exact d'écriture pour le tampon des fichiers mod
        	self.repTempFichiersMod = self.repTempEntree+'transcodage'+os.sep+'fichiers_mod'+os.sep
		# Création du chemin
		if os.path.isdir(self.repTempFichiersMod) is False: os.makedirs(self.repTempFichiersMod)
		# Epuration/elimination des fichiers tampon contenus dans le rep tampon
		for toutRepTemp in glob.glob(self.repTempFichiersMod+'*.*'): os.remove(toutRepTemp)

		#----------------------------------------------------------------------------------
		# Paramètres de la liste de combo: [(identifiant, nom entrée, commande FFmpeg, instance, extension), ...]
		#----------------------------------------------------------------------------------
		
		self.listeCombo = [
			('youtube_1080p_16/9_qual_normale', _(u'Transcodage YouTube (1920x1080, 16/9), qualité normale'), '-f mp4 -c:v libx264 -b:v 8000k -s 1920x1080 -aspect 16:9 -c:a libvo_aacenc -b:a 384k', 0, '.mp4'),
			('youtube_720p_16/9_qual_normale', _(u'Transcodage YouTube (1280x720, 16/9), qualité normale'), '-f mp4 -c:v libx264 -b:v 5000k -s 1280x720 -aspect 16:9 -c:a libvo_aacenc -b:a 384k', 1, '.mp4'),
			('youtube_480p_16/9_qual_normale', _(u'Transcodage YouTube (854x480, 16/9), qualité normale'), '-f mp4 -c:v libx264 -b:v 2500k -s 854x480 -aspect 16:9 -c:a libvo_aacenc -b:a 128k', 2, '.mp4'),
			('youtube_360p_16/9_qual_normale', _(u'Transcodage YouTube (640x360, 16/9), qualité normale'), '-f mp4 -c:v libx264 -b:v 1000k -s 640x360 -aspect 16:9 -c:a libvo_aacenc -b:a 128k', 3, '.mp4'),
			('youtube_768x576_4/3_qual_normale', _(u'Transcodage YouTube (768x576, 4/3), qualité normale'), '-f mp4 -c:v libx264 -b:v 2500k -s 768x576 -aspect 4:3 -c:a libvo_aacenc -b:a 128k', 4, '.mp4'),
			('youtube_512x384_4/3_qual_normale', _(u'Transcodage YouTube (512x384, 4/3), qualité normale'), '-f mp4 -c:v libx264 -b:v 2500k -s 512x384 -aspect 4:3 -c:a libvo_aacenc -b:a 128k', 5, '.mp4'),
			('youtube_1080p_16/9_qual_pro', _(u'Transcodage YouTube (1920x1080, 16/9), qualité pro'), '-f mp4 -c:v libx264 -b:v 50000k -s 1920x1080 -aspect 16:9 -c:a libvo_aacenc -b:a 384k', 6, '.mp4'),
			('youtube_720p_16/9_qual_pro', _(u'Transcodage YouTube (1280x720, 16/9), qualité pro'), '-f mp4 -c:v libx264 -b:v 30000k -s 1280x720 -aspect 16:9 -c:a libvo_aacenc -b:a 384k', 7, '.mp4'),
			('youtube_480p_16/9_qual_pro', _(u'Transcodage YouTube (854x480, 16/9), qualité pro'), '-f mp4 -c:v libx264 -b:v 15000k -s 854x480 -aspect 16:9 -c:a libvo_aacenc -b:a 384k', 8, '.mp4'),
			('youtube_360p_16/9_qual_pro', _(u'Transcodage YouTube (640x360, 16/9), qualité pro'), '-f mp4 -c:v libx264 -b:v 5000k -s 640x360 -aspect 16:9 -c:a libvo_aacenc -b:a 384k', 9, '.mp4'),
			('dailymotion_1080p_16/9_hd', _(u'Transcodage Dailymotion (1920x1080, 16/9), HD'), '-f mp4 -c:v libx264 -r 25 -b:v 4000k -s 1920x1080 -aspect 16:9 -c:a libvo_aacenc -b:a 256k', 10, '.mp4'),
			('dailymotion_720p_16/9_hd', _(u'Transcodage Dailymotion (1280x720, 16/9), HD'), '-f mp4 -c:v libx264 -r 25 -b:v 3000k -s 1280x720 -aspect 16:9 -c:a libvo_aacenc -b:a 256k', 11, '.mp4'),
			('dailymotion_480p_16/9', _(u'Transcodage Dailymotion (854x480, 16/9)'), '-f mp4 -c:v libx264 -r 25 -b:v 2000k -s 854x480 -aspect 16:9 -c:a libvo_aacenc -b:a 256k', 12, '.mp4'),
			('dailymotion_640x480_4/3_sd', _(u'Transcodage Dailymotion (640x480, 4/3), SD'), '-f mp4 -c:v libx264 -r 25 -b:v 1400k -s 640x480 -aspect 4:3 -c:a libvo_aacenc -b:a 256k', 13, '.mp4'),
			('vimeo_1080p_16/9_hd_30_img/sec', _(u'Transcodage Vimeo (1920x1080, 16/9), HD, 30 img/sec'), '-f mp4 -c:v libx264 -r 30 -b:v 5000k -s 1920x1080 -aspect 16:9 -c:a libvo_aacenc -b:a 320k', 14, '.mp4'),
			('vimeo_1080p_16/9_hd_25_img/sec', _(u'Transcodage Vimeo (1920x1080, 16/9), HD, 25 img/sec'), '-f mp4 -c:v libx264 -r 25 -b:v 5000k -s 1920x1080 -aspect 16:9 -c:a libvo_aacenc -b:a 320k', 15, '.mp4'),
			('vimeo_720p_16/9_hd_30_img/sec', _(u'Transcodage Vimeo (1280x720, 16/9), HD, 30 img/sec'), '-f mp4 -c:v libx264 -r 30 -b:v 5000k -s 1280x720 -aspect 16:9 -c:a libvo_aacenc -b:a 320k', 16, '.mp4'),
			('vimeo_720p_16/9_hd_25_img/sec', _(u'Transcodage Vimeo (1280x720, 16/9), HD, 25 img/sec'), '-f mp4 -c:v libx264 -r 25 -b:v 5000k -s 1280x720 -aspect 16:9 -c:a libvo_aacenc -b:a 320k', 17, '.mp4'),
			('vimeo_360p_16/9_30_img/sec', _(u'Transcodage Vimeo (640×360, 16/9), 30 img/sec'), '-f mp4 -c:v libx264 -r 30 -b:v 2500k -s 640x360 -aspect 16:9 -c:a libvo_aacenc -b:a 128k', 18, '.mp4'),
			('vimeo_360p_16/9_25_img/sec', _(u'Transcodage Vimeo (640×360, 16/9), 25 img/sec'), '-f mp4 -c:v libx264 -r 25 -b:v 2500k -s 640x360 -aspect 16:9 -c:a libvo_aacenc -b:a 128k', 19, '.mp4'),
			('vimeo_640x480_4/3_30_img/sec', _(u'Transcodage Vimeo (640x480, 4/3), 30 img/sec'), '-f mp4 -c:v libx264 -r 30 -b:v 2500k -s 640x480 -aspect 4:3 -c:a libvo_aacenc -b:a 128k', 18, '.mp4'),
			('vimeo_640x480_4/3_25_img/sec', _(u'Transcodage Vimeo (640x480, 4/3), 25 img/sec'), '-f mp4 -c:v libx264 -r 25 -b:v 2500k -s 640x480 -aspect 4:3 -c:a libvo_aacenc -b:a 128k', 19, '.mp4'),
			('webm_3840x2160', _(u'Modèle WebM 4K (3840x2160, 16/9)'), '-s 3840x2160 -aspect 16:9 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 20, '.webm'),
			('webm_4096x2160', _(u'Modèle WebM 4K Cin (4096x2160)'), '-s 4096x2160 -aspect 1.8962962962962964 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 21, '.webm'),
			('webm_4096x1716', _(u'Modèle WebM 4K Cin (4096x1716)'), '-s 4096x1716 -aspect 2.386946386946387 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 22, '.webm'),
			('webm_3996x2160', _(u'Modèle WebM 4K Cin (3996x2160)'), '-s 3996x2160 -aspect 1.85 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 23, '.webm'),
			('webm_2716x1524', _(u'Modèle WebM 2.7K (2716x1524)'), '-s 2716x1524 -aspect 1.7821522309711286 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 24, '.webm'),
			('webm_2716x1440', _(u'Modèle WebM 2.7K Cin (2716x1440)'), '-s 2716x1440 -aspect 1.886111111111111 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 25, '.webm'),
			('webm_2048x1080', _(u'Modèle WebM 2K Cin (2048x1080)'), '-s 2048x1080 -aspect 1.8962962962962964 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 26, '.webm'),
			('webm_2048x858', _(u'Modèle WebM 2K Cin (2048x858)'), '-s 2048x858 -aspect 2.386946386946387 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 27, '.webm'),
			('webm_1998x1080', _(u'Modèle WebM 2K Cin (1998x1080)'), '-s 1998x1080 -aspect 1.85 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 28, '.webm'),
			('webm_1440p_4_3', _(u'Modèle WebM (1920x1440, 4/3)'), '-s 1920x1440 -aspect 4:3 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 29, '.webm'),
			('webm_1080p_16/9', _(u'Modèle WebM (1920x1080, 16/9)'), '-s 1920x1080 -aspect 16:9 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 30, '.webm'),
			('webm_720p_16/9', _(u'Modèle WebM (1280x720, 16/9)'), '-s 1280x720 -aspect 16:9 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 31, '.webm'),
			('webm_480p_16/9', _(u'Modèle WebM (854x480, 16/9)'), '-s 854x480 -aspect 16:9 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 32, '.webm'),
			('webm_360p_16/9', _(u'Modèle WebM (640x360, 16/9)'), '-s 640x360 -aspect 16:9 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 33, '.webm'),
			('webm_768x576_4/3', _(u'Modèle WebM (768x576, 4/3)'), '-s 768x576 -aspect 4:3 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 34, '.webm'),
			('webm_512x384_4/3', _(u'Modèle WebM (512x384, 4/3)'), '-s 512x384 -aspect 4:3 -codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm', 35, '.webm'),
			]

		#---------------------------
		# Dérivation de la classe
		#---------------------------
		Base_EncodageFiltre.__init__(self, titre=_(u"Transcodage: Pour le web"),
									 videofilters=["*.avi", "*.mov", "*.mpg", "*.mpeg", "*.mjpeg", "*.flv", "*.mp4", "*.h264", "*.dv", "*.vob", "*.mkv", "*.mod", "*.ogg", "*.ogv", "*.ogm"] )


	def appliquer(self, nomSortie=None, ouvert=1):
		"""Appel du moteur du cadre"""

		# quel est l'index du dernier item sélectionné de la boîte de combo?
		index=self.combo.currentIndex()
		if not nomSortie:

			## On utilise la nouvelle interface de récupération des vidéos
			# Récupération du chemin source
			chemin=unicode(self.getFile())
			# suffix du codec actif
			suffix=self.listeCombo[index][4]
			if suffix=='':
				suffix=os.path.splitext(chemin)[1]
				
			# ------------------------------------------------------------------- #
			# Gestion des fichiers mod (extension .mod). Ce sont (apparemment)
			# des fichiers mpeg avec une extension .mod. Les fichiers en question 
			# ont juste besoin d'être renommés avec une extension .mpg avant le 
			# traitement.
			# ------------------------------------------------------------------- #
			ext_chargee = os.path.splitext(chemin)[1]
			nom_fich_sans_ext = os.path.splitext(os.path.basename(chemin))[0]
			if ext_chargee in ['.mod', '.MOD']:
				# Copie du fichier concerné dans le rep tampon et renommage avec ext .mpg
				shutil.copy(chemin, self.repTempFichiersMod+nom_fich_sans_ext + '.mpg')
				chemin = unicode(self.repTempFichiersMod + nom_fich_sans_ext + '.mpg')
			
			# suffix du codec actif
                        saveDialog = EkdSaveDialog(self, mode="video", suffix=suffix, title=_(u"Sauver"))
			cheminFichierEnregistrerVideo = saveDialog.getFile()
			
		else: # module séquentiel
			chemin = cheminFichierEnregistrerVideo = nomSortie

		if not cheminFichierEnregistrerVideo:
			return

		# quel est l'index du dernier item sélectionné de la boîte de combo?
		#print 'index', index
		EkdPrint('index ' + str(index))
		# identifiant du codec actif
		idCodec = self.listeCombo[index][0]
		#print 'idCodec', idCodec
		EkdPrint('idCodec ' + idCodec)
		
		if idCodec in ['webm_3840x2160', 'webm_4096x2160', 'webm_4096x1716', 'webm_3996x2160', 'webm_2716x1524', 'webm_2716x1440']:
			messMplayerVidGrdTail=QMessageBox(self)
			messMplayerVidGrdTail.setText(_(u"<p>Vous allez transcoder votre vidéo avec une très haute définition ...</p><p>A la fin du traitement, <b>il se peut que la vidéo ne puisse pas être lue dans l'onglet: Visionner vidéo et que vous ayez des messages d'erreur à répétition</b> (cela est du à la configuration de votre carte graphique en association à MPlayer, le lecteur d'EKD).</p><p><b>Si vous rencontrez ce problème, sachez que votre vidéo a bien été convertie mais ne pourra pas être lue par les lecteurs présents sur votre ordinateur.</b></p>"))
			messMplayerVidGrdTail.setWindowTitle(_(u"Avertissement"))
			messMplayerVidGrdTail.setIcon(QMessageBox.Information)
			messMplayerVidGrdTail.exec_()

		ffmpeg = WidgetFFmpeg(idCodec, chemin, cheminFichierEnregistrerVideo, valeurNum = self.listeCombo[index][2], laisserOuvert=ouvert)
		ffmpeg.setWindowTitle(_(u"Transcodage vidéo - Pour le web"))
		ffmpeg.exec_()

		# Condition pour détection windows
		if os.name == 'nt':
			self.lstFichiersSortie =  cheminFichierEnregistrerVideo # pour la boite de dialogue de comparaison
		# Condition pour détection Linux
		elif os.name in ['posix', 'mac']:
			self.lstFichiersSortie =  cheminFichierEnregistrerVideo # pour la boite de dialogue de comparaison
		self.radioConvert.setChecked(True)
		self.radioSource.setEnabled(True)
		self.radioSource.setChecked(False)
		self.radioConvert.setEnabled(True)
		self.boutCompare.setEnabled(True)
		### Information à l'utilisateur
		self.infoLog(None, chemin, None, cheminFichierEnregistrerVideo)

		return cheminFichierEnregistrerVideo # module séquentiel

	def tamponAction(self) :
		#print u"Message différent"
		EkdPrint(u"Message différent")


	def afficherAide(self):
		"""Boîte de dialogue de l'aide du cadre"""

		super(AnimationEncodageWeb,self).afficherAide(_(u"<p><b>Sous la dénomination 'Pour le web', vous pouvez ici transcoder vos vidéos afin qu'elles soient publiables sur internet et plus précisément sur Google (Google video), Youtube ou sur Dailymotion.</b></p><p>Dans l'onglet <b>'Vidéo(s) source'</b> cliquez sur le bouton <b>Ajouter</b>, une boîte de dialogue apparaît, sur la partie gauche sélectionnez le répertoire (au besoin dépliez les sous-répertoires), allez chercher la/les vidéo(s). Si vous voulez sélectionner plusieurs vidéos d'un coup, maintenez la touche <b>CTRL</b> (ou <b>SHIFT</b>) du clavier enfoncée (tout en sélectionnant vos vidéos), cliquez sur <b>Ajouter</b>.</p><p>Vous pouvez dès lors sélectionner une vidéo dans la liste et la visionner (par le bouton juste à la droite de cette liste), vous noterez que vous pouvez visionner la vidéo en quatre tiers, en seize neuvième ou avec les proportions d'origine de la vidéo (w;h). De même si vous le désirez, vous pouvez obtenir des informations complètes sur la vidéo sélectionnée, et ce par le bouton <b>'Infos'</b> (en bas).</p><p>Dans l'onglet <b>'Réglages'</b> sélectionnez le type d'encodage que vous voulez faire.</p><p>Une fois tout ceci fait, cliquez sur le bouton <b>'Appliquer'</b>, sélectionnez le répertoire de sauvegarde, indiquez votre <b>'Nom de fichier'</b>, cliquez sur le bouton <b>'Enregistrer'</b> et attendez le temps de la conversion. A la fin cliquez sur le bouton <b>'Voir les informations d'encodage'</b> et fermez cette dernière fenêtre après avoir vu les informations en question.</p><p>Dans l'onglet <b>'Visionner vidéo'</b> vous pouvez visionner le résultat (avant la conversion) en sélectionnant <b>'vidéo(s) source(s)'</b>, après la conversion <b>'vidéo convertie'</b> ou bien encore les deux en même temps, en cliquant sur le bouton <b>'Comparateur de vidéos'</b>.</p><p>L'onglet <b>'Infos'</b> vous permet de voir les vidéos chargées (avec leurs chemins exacts) avant et après conversion.</p>"))
