#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
# On importe le strict nécessaire
from PyQt4.QtCore import SIGNAL, SLOT, Qt
from PyQt4.QtGui import QWidget, QSpinBox, QSlider, QHBoxLayout, QGridLayout, QLabel, QCheckBox
# Utilisation de EkdConfig
from moteur_modules_common.EkdConfig import EkdConfig

###############
from moteur_modules_common.EkdCompatiblePython2Et3 import EkdPrint
###############


class Base(object):
	#-------------------------------------
	# Fonctions communes au module Image
	#-------------------------------------

	def getRepSource(self, config):
		"Renvoie le répertoire source qui sera utilisée dans la boite de dialogue de fichiers"
		try:
			repEntree = config.get('general','image_input_path').decode("UTF8")
		except Exception, e:
			repEntree = '~'
		if not os.path.exists(repEntree):
			repEntree = '~'

		return os.path.expanduser(repEntree)


	def setRepSource(self, config, cheminEntree):
		"Sauvegarde le répertoire source qui sera utilisée dans la boite de dialogue de fichiers"
		if isinstance(cheminEntree, list):
			if len(cheminEntree)==0: return
			config.set('general','image_input_path',os.path.dirname(cheminEntree[0]).encode("UTF8"))

		elif isinstance(cheminEntree, unicode):
			if not cheminEntree: return
			config.set('general','image_input_path',os.path.dirname(cheminEntree).encode("UTF8"))


	def valeurComboIni(self, comboBox, config, idSection, optionConfig):
		"Affiche au lancement du logiciel l'entrée de la boite de combo inscrite dans un fichier de configuration"
		try:
			valeur = config.get(idSection, optionConfig)
		except:
			#print "Pas de valeur dans la config pour : ", idSection, optionConfig
			EkdPrint(u"Pas de valeur dans la config pour : %s %s" % (idSection, optionConfig))
		else:
			codec_trouve = False
			for i in range(comboBox.count()):
				if comboBox.itemData(i).toString() == valeur:
					indice = i
					codec_trouve = True
			if not codec_trouve: indice = 0
			comboBox.setCurrentIndex(indice)


	def printSection(self, idSection):
		"""Affichage dans le terminal de la section initialisée"""
		#print '\n'+'-'*30+'\n| '+idSection+'\n'+'-'*30+'\n'
		EkdPrint('\n%s\n| %s\n%s\n' % ('-'*30, idSection, '-'*30))


class SpinSlider(QWidget):
	"Boite de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	def __init__(self, debut, fin, defaut, optionConfig=None, parent=None):
		QWidget.__init__(self)

		self.spin = QSpinBox()
		self.slider = QSlider(Qt.Horizontal)
		self.spin.setRange(debut, fin)
		self.slider.setRange(debut, fin)

		valeur = None
		if optionConfig:
			self.config = parent.config
			self.idSection = parent.idSection
			self.optionConfig = optionConfig
			try :
				valeur = self.config.get(self.idSection, self.optionConfig)
			except:
				valeur = defaut
				self.config.set(self.idSection, self.optionConfig, valeur)
		erreur = False
		if valeur:
			try: valeur = int(valeur)
			except ValueError: erreur = True
		if not erreur and debut <= valeur <= fin:
			self.spin.setValue(valeur)
			self.slider.setValue(valeur)
		else:
			self.spin.setValue(defaut)
			self.slider.setValue(defaut)
		if optionConfig: self.connect(self.spin, SIGNAL("valueChanged(int)"), self.sauveConfig)
		self.connect(self.spin, SIGNAL("valueChanged(int)"), self.slider,
						SLOT("setValue(int)"))
		self.connect(self.slider, SIGNAL("valueChanged(int)"), self.spin,
						SLOT("setValue(int)"))
		self.connect(self.spin, SIGNAL("valueChanged(int)"), self.changed)
		hbox = QHBoxLayout(self)
		hbox.addWidget(self.spin)
		hbox.addWidget(self.slider)
		hbox.setMargin(0) # pour un meilleur alignement dans un grid

	def changed(self, i):
		self.emit(SIGNAL("valueChanged(int)"), i)

	def sauveConfig(self, i):
		self.config.set(self.idSection, self.optionConfig, str(i))

	def value(self):
		return self.spin.value()

	def setValue(self, i):
		self.spin.setValue(i)
		self.slider.setValue(i)


class SpinSliders(QWidget):
	"Boites de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	def __init__(self, parent,
				debut1, fin1, defaut1, txt1, optionConfig1,
				debut2=None, fin2=None, defaut2=None, txt2=None, optionConfig2=None,
				debut3=None, fin3=None, defaut3=None, txt3=None, optionConfig3=None,
				):
		QWidget.__init__(self)
		grid = QGridLayout(self)
		self.spin = SpinSlider(debut1, fin1, defaut1, optionConfig1, parent)
		self.spin1= self.spin
		grid.addWidget(self.spin,0,0)
		if txt1: grid.addWidget(QLabel(txt1),0,1)
		if debut2:
			self.spin2 = SpinSlider(debut2, fin2, defaut2, optionConfig2, parent)
			grid.addWidget(self.spin2,1,0)
			if txt2: grid.addWidget(QLabel(txt2),1,1)
		if debut3:
			self.spin3 = SpinSlider(debut3, fin3, defaut3, optionConfig3, parent)
			grid.addWidget(self.spin3,2,0)
			if txt3: grid.addWidget(QLabel(txt3),2,1)
		grid.setAlignment(Qt.AlignCenter)
		grid.setMargin(0)


class SpinSliders20Possibles(QWidget):
	"Boites de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	"Modele pour 20 boites de spin possibles (ce qui laisse pas mal de possibilités de réglages)"

	def __init__(self, parent,
				debut1, fin1, defaut1, txt1, optionConfig1,
				debut2=None, fin2=None, defaut2=None, txt2=None, optionConfig2=None,
				debut3=None, fin3=None, defaut3=None, txt3=None, optionConfig3=None,
				debut4=None, fin4=None, defaut4=None, txt4=None, optionConfig4=None,
				debut5=None, fin5=None, defaut5=None, txt5=None, optionConfig5=None,
				debut6=None, fin6=None, defaut6=None, txt6=None, optionConfig6=None,
				debut7=None, fin7=None, defaut7=None, txt7=None, optionConfig7=None,
				debut8=None, fin8=None, defaut8=None, txt8=None, optionConfig8=None,
				debut9=None, fin9=None, defaut9=None, txt9=None, optionConfig9=None,
				debut10=None, fin10=None, defaut10=None, txt10=None, optionConfig10=None,
				debut11=None, fin11=None, defaut11=None, txt11=None, optionConfig11=None,
				debut12=None, fin12=None, defaut12=None, txt12=None, optionConfig12=None,
				debut13=None, fin13=None, defaut13=None, txt13=None, optionConfig13=None,
				debut14=None, fin14=None, defaut14=None, txt14=None, optionConfig14=None,
				debut15=None, fin15=None, defaut15=None, txt15=None, optionConfig15=None,
				debut16=None, fin16=None, defaut16=None, txt16=None, optionConfig16=None,
				debut17=None, fin17=None, defaut17=None, txt17=None, optionConfig17=None,
				debut18=None, fin18=None, defaut18=None, txt18=None, optionConfig18=None,
				debut19=None, fin19=None, defaut19=None, txt19=None, optionConfig19=None,
				debut20=None, fin20=None, defaut20=None, txt20=None, optionConfig20=None,				
				):
		QWidget.__init__(self)
		grid = QGridLayout(self)
		self.spin = SpinSlider(debut1, fin1, defaut1, optionConfig1, parent)
		self.spin1= self.spin
		grid.addWidget(self.spin,0,0)
		if txt1: grid.addWidget(QLabel(txt1),0,1)
		if debut2:
			self.spin2 = SpinSlider(debut2, fin2, defaut2, optionConfig2, parent)
			grid.addWidget(self.spin2,1,0)
			if txt2: grid.addWidget(QLabel(txt2),1,1)
		if debut3:
			self.spin3 = SpinSlider(debut3, fin3, defaut3, optionConfig3, parent)
			grid.addWidget(self.spin3,2,0)
			if txt3: grid.addWidget(QLabel(txt3),2,1)
		if debut4:
			self.spin4 = SpinSlider(debut4, fin4, defaut4, optionConfig4, parent)
			grid.addWidget(self.spin4,3,0)
			if txt4: grid.addWidget(QLabel(txt4),3,1)
		if debut5:
			self.spin5 = SpinSlider(debut5, fin5, defaut5, optionConfig5, parent)
			grid.addWidget(self.spin5,4,0)
			if txt5: grid.addWidget(QLabel(txt5),4,1)
		if debut6:
			self.spin6 = SpinSlider(debut6, fin6, defaut6, optionConfig6, parent)
			grid.addWidget(self.spin6,5,0)
			if txt6: grid.addWidget(QLabel(txt6),5,1)
		if debut7:
			self.spin7 = SpinSlider(debut7, fin7, defaut7, optionConfig7, parent)
			grid.addWidget(self.spin7,6,0)
			if txt7: grid.addWidget(QLabel(txt7),6,1)
		if debut8:
			self.spin8 = SpinSlider(debut8, fin8, defaut8, optionConfig8, parent)
			grid.addWidget(self.spin8,7,0)
			if txt8: grid.addWidget(QLabel(txt8),7,1)
		if debut9:
			self.spin9 = SpinSlider(debut9, fin9, defaut9, optionConfig9, parent)
			grid.addWidget(self.spin9,8,0)
			if txt9: grid.addWidget(QLabel(txt9),8,1)
		if debut10:
			self.spin10 = SpinSlider(debut10, fin10, defaut10, optionConfig10, parent)
			grid.addWidget(self.spin10,9,0)
			if txt10: grid.addWidget(QLabel(txt10),9,1)
		if debut11:
			self.spin11 = SpinSlider(debut11, fin11, defaut11, optionConfig11, parent)
			grid.addWidget(self.spin11,10,0)
			if txt11: grid.addWidget(QLabel(txt11),10,1)
		if debut12:
			self.spin12 = SpinSlider(debut12, fin12, defaut12, optionConfig12, parent)
			grid.addWidget(self.spin12,11,0)
			if txt12: grid.addWidget(QLabel(txt12),11,1)
		if debut13:
			self.spin13 = SpinSlider(debut13, fin13, defaut13, optionConfig13, parent)
			grid.addWidget(self.spin13,12,0)
			if txt13: grid.addWidget(QLabel(txt13),12,1)
		if debut14:
			self.spin14 = SpinSlider(debut14, fin14, defaut14, optionConfig14, parent)
			grid.addWidget(self.spin14,13,0)
			if txt14: grid.addWidget(QLabel(txt14),13,1)
		if debut15:
			self.spin15 = SpinSlider(debut15, fin15, defaut15, optionConfig15, parent)
			grid.addWidget(self.spin15,14,0)
			if txt15: grid.addWidget(QLabel(txt15),14,1)
		if debut16:
			self.spin16 = SpinSlider(debut16, fin16, defaut16, optionConfig16, parent)
			grid.addWidget(self.spin16,15,0)
			if txt16: grid.addWidget(QLabel(txt16),15,1)
		if debut17:
			self.spin17 = SpinSlider(debut17, fin17, defaut17, optionConfig17, parent)
			grid.addWidget(self.spin17,16,0)
			if txt17: grid.addWidget(QLabel(txt17),16,1)
		if debut18:
			self.spin18 = SpinSlider(debut18, fin18, defaut18, optionConfig18, parent)
			grid.addWidget(self.spin18,17,0)
			if txt18: grid.addWidget(QLabel(txt18),17,1)
		if debut19:
			self.spin19 = SpinSlider(debut19, fin19, defaut19, optionConfig19, parent)
			grid.addWidget(self.spin19,18,0)
			if txt19: grid.addWidget(QLabel(txt19),18,1)
		if debut20:
			self.spin20 = SpinSlider(debut20, fin20, defaut20, optionConfig20, parent)
			grid.addWidget(self.spin20,19,0)
			if txt20: grid.addWidget(QLabel(txt20),19,1)
		grid.setAlignment(Qt.AlignCenter)
		grid.setMargin(0)


"""
class SpinSliders20Possibles(QWidget):
	"Boites de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	"Modele pour (l'instant) de 10 boites de spin possibles (ce qui laisse pas mal"
	"de possibilités de réglages)"

	def __init__(self, parent, *args):

		#QWidget.__init__(self, parent)
		super(SpinSliders20Possibles, self).__init__(parent)
		
		# Nombre d'arguments au total dans l'appel de la classe. Exemple:
		# stacked_vieille_photo = SpinSliders(self,
		# 1,10,4, _(u"Taille en largeur de l'ombre"), 'taill_larg_ombre_vieil_photo',
		# ..., 1,90,10, _(u"Rotation (en degrés) de la photo"), 'rot_photo_vieil_photo')
		nbre_arguments = 5
		
		# Comprehension list pour collecter les infos des données des réglages. La liste est 
		# une liste de tuples de la forme: [(index, donnée 1), (index, donnée 2), ...]. Exemple:
		# [(0, 1), (0, 10), (0, 1), (0, u'Nettet\xe9 du trait'), (0, 'nett_trait_bord_tendu'), 
		# ..., (5, 128), (5, 255), (5, 255), (5, u'Couleurs, fin'), (5, 'coul_fin_img_bord_tendu')]
		clist1 = [(y/nbre_arguments, arg) for y, arg in enumerate(args)]
		
		# Hachage des données par tranches de 5 (car nbre_arguments est défini à 5). Calcul des multiples de 5.
		clist2 = [clist1[z:z+nbre_arguments] for z in range(len(clist1)) if z % nbre_arguments == 0]

		grid = QGridLayout(self)
		# Boucle principale pour mise en place des spin
		for i1 in range(len(clist2)):
			# Attribution des données suivant les index
			debut = clist2[i1][0][1]
			fin = clist2[i1][1][1]
			defaut = clist2[i1][2][1]
			optionConfig = clist2[i1][3][1]
			'''
			### NE FONTIONNE PAS ##########################################################
			# Attribution des données au différents spin
			self.spin[i1+1] = SpinSlider(debut, fin, defaut, optionConfig, parent)
			# Attribution du spin (de tous les spin) suivant les index
			grid.addWidget(self.spin[i1+1], clist2[i1][0][0],0)
			# Attribution du texte du spin (des textes de tous les spin) suivant les index
			grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			# Traceback (most recent call last):
			# File "ekd_gui.py", line 1929, in <module>
			# main = Gui_EKD()
			# File "ekd_gui.py", line 1151, in __init__
			# Image_FiltresImage(self.statusBar(), self.frameGeometry), _(u"Filtres"),\
			# File "/home/user/ekd_toto/gui_modules_image/filtres_images/image_filtres_image.py", line 480, in __init__
			# 1,90,10, _(u"Rotation (en degrés) de la photo"), 'rot_photo_polaroid')
			# File "/home/user/ekd_toto/gui_modules_image/image_base.py", line 294, in __init__
			# self.spin[i1+1] = SpinSlider(debut, fin, defaut, optionConfig, parent)
			# AttributeError: 'SpinSliders20Possibles' object has no attribute 'spin'
			###############################################################################
			'''
			if i1 == 0:
				self.spin1 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin1, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 1:
				self.spin2 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin2, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 2:
				self.spin3 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin3, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 3:
				self.spin4 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin4, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 4:
				self.spin5 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin5, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 5:
				self.spin6 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin6, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 6:
				self.spin7 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin7, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 7:
				self.spin8 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin8, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 8:
				self.spin9 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin9, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
			elif i1 == 9:
				self.spin10 = SpinSlider(debut, fin, defaut, optionConfig, parent)
				grid.addWidget(self.spin10, clist2[i1][0][0],0)
				grid.addWidget(QLabel(optionConfig), clist2[i1][3][0],1)
		grid.setAlignment(Qt.AlignCenter)
		grid.setMargin(0)
"""

class SpinSliders20PossiblesDebutFin(QWidget):
	"Boites de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	"Modele pour 20 boites de spin possibles (ce qui laisse pas mal de possibilités de réglages)."
	"La particularité de ce modèle est qu'il permet d'apposer des réglages en début de traitement"
	"et en fin de traitement (pendant le traitement un calcul se fera entre la 1ère image et la"
	"dernière).Cela aura pour conséquence un changement visible dans le filtre (et suivant les" 
	"réglages lui-même entre la 1ère image et la dernière de la série --> réglages de traitement"
	"début ... réglages de traitement fin)."
	def __init__(self, parent,
				debut1_1=None, fin1_1=None, defaut1_1=None, txt1_1=None, optionConfig1_1=None, 
				debut1_2=None, fin1_2=None, defaut1_2=None, txt1_2=None, optionConfig1_2=None,
				debut2_1=None, fin2_1=None, defaut2_1=None, txt2_1=None, optionConfig2_1=None,
				debut2_2=None, fin2_2=None, defaut2_2=None, txt2_2=None, optionConfig2_2=None,
				debut3_1=None, fin3_1=None, defaut3_1=None, txt3_1=None, optionConfig3_1=None,
				debut3_2=None, fin3_2=None, defaut3_2=None, txt3_2=None, optionConfig3_2=None,
				debut4_1=None, fin4_1=None, defaut4_1=None, txt4_1=None, optionConfig4_1=None,
				debut4_2=None, fin4_2=None, defaut4_2=None, txt4_2=None, optionConfig4_2=None,
				debut5_1=None, fin5_1=None, defaut5_1=None, txt5_1=None, optionConfig5_1=None,
				debut5_2=None, fin5_2=None, defaut5_2=None, txt5_2=None, optionConfig5_2=None,
				debut6_1=None, fin6_1=None, defaut6_1=None, txt6_1=None, optionConfig6_1=None,
				debut6_2=None, fin6_2=None, defaut6_2=None, txt6_2=None, optionConfig6_2=None,
				debut7_1=None, fin7_1=None, defaut7_1=None, txt7_1=None, optionConfig7_1=None,
				debut7_2=None, fin7_2=None, defaut7_2=None, txt7_2=None, optionConfig7_2=None,
				debut8_1=None, fin8_1=None, defaut8_1=None, txt8_1=None, optionConfig8_1=None,
				debut8_2=None, fin8_2=None, defaut8_2=None, txt8_2=None, optionConfig8_2=None,
				debut9_1=None, fin9_1=None, defaut9_1=None, txt9_1=None, optionConfig9_1=None,
				debut9_2=None, fin9_2=None, defaut9_2=None, txt9_2=None, optionConfig9_2=None,
				debut10_1=None, fin10_1=None, defaut10_1=None, txt10_1=None, optionConfig10_1=None,
				debut10_2=None, fin10_2=None, defaut10_2=None, txt10_2=None, optionConfig10_2=None,
				debut11_1=None, fin11_1=None, defaut11_1=None, txt11_1=None, optionConfig11_1=None, 
				debut11_2=None, fin11_2=None, defaut11_2=None, txt11_2=None, optionConfig11_2=None,
				debut12_1=None, fin12_1=None, defaut12_1=None, txt12_1=None, optionConfig12_1=None, 
				debut12_2=None, fin12_2=None, defaut12_2=None, txt12_2=None, optionConfig12_2=None,
				debut13_1=None, fin13_1=None, defaut13_1=None, txt13_1=None, optionConfig13_1=None, 
				debut13_2=None, fin13_2=None, defaut13_2=None, txt13_2=None, optionConfig13_2=None,
				debut14_1=None, fin14_1=None, defaut14_1=None, txt14_1=None, optionConfig14_1=None, 
				debut14_2=None, fin14_2=None, defaut14_2=None, txt14_2=None, optionConfig14_2=None,
				debut15_1=None, fin15_1=None, defaut15_1=None, txt15_1=None, optionConfig15_1=None, 
				debut15_2=None, fin15_2=None, defaut15_2=None, txt15_2=None, optionConfig15_2=None,
				debut16_1=None, fin16_1=None, defaut16_1=None, txt16_1=None, optionConfig16_1=None, 
				debut16_2=None, fin16_2=None, defaut16_2=None, txt16_2=None, optionConfig16_2=None,
				debut17_1=None, fin17_1=None, defaut17_1=None, txt17_1=None, optionConfig17_1=None, 
				debut17_2=None, fin17_2=None, defaut17_2=None, txt17_2=None, optionConfig17_2=None,
				debut18_1=None, fin18_1=None, defaut18_1=None, txt18_1=None, optionConfig18_1=None, 
				debut18_2=None, fin18_2=None, defaut18_2=None, txt18_2=None, optionConfig18_2=None,
				debut19_1=None, fin19_1=None, defaut19_1=None, txt19_1=None, optionConfig19_1=None, 
				debut19_2=None, fin19_2=None, defaut19_2=None, txt19_2=None, optionConfig19_2=None,
				debut20_1=None, fin20_1=None, defaut20_1=None, txt20_1=None, optionConfig20_1=None, 
				debut20_2=None, fin20_2=None, defaut20_2=None, txt20_2=None, optionConfig20_2=None,				
				):
		
		QWidget.__init__(self)

		grid = QGridLayout(self)
		# Case à cocher pour opérer le traitement du début
		self.labelCheckBoxTraitDebut = QLabel(_(u"Réglages pour <b>traitement début</b> "))
		grid.addWidget(self.labelCheckBoxTraitDebut,0,0)
		self.checkBoxTraitDebut = QCheckBox(self)
		self.checkBoxTraitDebut.setChecked(True)
		grid.addWidget(self.checkBoxTraitDebut,0,1)
		self.connect(self.checkBoxTraitDebut, SIGNAL("clicked(bool)"), self.majChoixTraitementDebut)
		# Case à cocher pour opérer le traitement de fin
		self.labelCheckBoxTraitFin = QLabel(_(u"Réglages pour <b>traitement fin</b> "))
		grid.addWidget(self.labelCheckBoxTraitFin,0,2)
		self.checkBoxTraitFin = QCheckBox(self)
		self.checkBoxTraitFin.setChecked(False)
		grid.addWidget(self.checkBoxTraitFin,0,3)
		self.connect(self.checkBoxTraitFin, SIGNAL("clicked(bool)"), self.majChoixTraitementFin)
			
		# Premier spin et Slider
		if debut1_1:
			self.spin1_1 = SpinSlider(debut1_1, fin1_1, defaut1_1, optionConfig1_1, parent)
			grid.addWidget(self.spin1_1,1,0)
			if txt1_1: grid.addWidget(QLabel(txt1_1),1,1)
		if debut1_2:
			self.spin1_2 = SpinSlider(debut1_2, fin1_2, defaut1_2, optionConfig1_2, parent)
			grid.addWidget(self.spin1_2,1,2)
			if txt1_2: grid.addWidget(QLabel(txt1_2),1,3)
		# Les spin et slider suivants
		if debut2_1:
			self.spin2_1 = SpinSlider(debut2_1, fin2_1, defaut2_1, optionConfig2_1, parent)
			grid.addWidget(self.spin2_1,2,0)
			if txt2_1: grid.addWidget(QLabel(txt2_1),2,1)
		if debut2_2:
			self.spin2_2 = SpinSlider(debut2_2, fin2_2, defaut2_2, optionConfig2_2, parent)
			grid.addWidget(self.spin2_2,2,2)
			if txt2_2: grid.addWidget(QLabel(txt2_2),2,3)
		if debut3_1:
			self.spin3_1 = SpinSlider(debut3_1, fin3_1, defaut3_1, optionConfig3_1, parent)
			grid.addWidget(self.spin3_1,3,0)
			if txt3_1: grid.addWidget(QLabel(txt3_1),3,1)
		if debut3_2:
			self.spin3_2 = SpinSlider(debut3_2, fin3_2, defaut3_2, optionConfig3_2, parent)
			grid.addWidget(self.spin3_2,3,2)
			if txt3_2: grid.addWidget(QLabel(txt3_2),3,3)
		if debut4_1:
			self.spin4_1 = SpinSlider(debut4_1, fin4_1, defaut4_1, optionConfig4_1, parent)
			grid.addWidget(self.spin4_1,4,0)
			if txt4_1: grid.addWidget(QLabel(txt4_1),4,1)
		if debut4_2:
			self.spin4_2 = SpinSlider(debut4_2, fin4_2, defaut4_2, optionConfig4_2, parent)
			grid.addWidget(self.spin4_2,4,2)
			if txt4_2: grid.addWidget(QLabel(txt4_2),4,3)
		if debut5_1:
			self.spin5_1 = SpinSlider(debut5_1, fin5_1, defaut5_1, optionConfig5_1, parent)
			grid.addWidget(self.spin5_1,5,0)
			if txt5_1: grid.addWidget(QLabel(txt5_1),5,1)
		if debut5_2:
			self.spin5_2 = SpinSlider(debut5_2, fin5_2, defaut5_2, optionConfig5_2, parent)
			grid.addWidget(self.spin5_2,5,2)
			if txt5_2: grid.addWidget(QLabel(txt5_2),5,3)
		if debut6_1:
			self.spin6_1 = SpinSlider(debut6_1, fin6_1, defaut6_1, optionConfig6_1, parent)
			grid.addWidget(self.spin6_1,6,0)
			if txt6_1: grid.addWidget(QLabel(txt6_1),6,1)
		if debut6_2:
			self.spin6_2 = SpinSlider(debut6_2, fin6_2, defaut6_2, optionConfig6_2, parent)
			grid.addWidget(self.spin6_2,6,2)
			if txt6_2: grid.addWidget(QLabel(txt6_2),6,3)
		if debut7_1:
			self.spin7_1 = SpinSlider(debut7_1, fin7_1, defaut7_1, optionConfig7_1, parent)
			grid.addWidget(self.spin7_1,7,0)
			if txt7_1: grid.addWidget(QLabel(txt7_1),7,1)
		if debut7_2:
			self.spin7_2 = SpinSlider(debut7_2, fin7_2, defaut7_2, optionConfig7_2, parent)
			grid.addWidget(self.spin7_2,7,2)
			if txt7_2: grid.addWidget(QLabel(txt7_2),7,3)
		if debut8_1:
			self.spin8_1 = SpinSlider(debut8_1, fin8_1, defaut8_1, optionConfig8_1, parent)
			grid.addWidget(self.spin8_1,8,0)
			if txt8_1: grid.addWidget(QLabel(txt8_1),8,1)
		if debut8_2:
			self.spin8_2 = SpinSlider(debut8_2, fin8_2, defaut8_2, optionConfig8_2, parent)
			grid.addWidget(self.spin8_2,8,2)
			if txt8_2: grid.addWidget(QLabel(txt8_2),8,3)
		if debut9_1:
			self.spin9_1 = SpinSlider(debut9_1, fin9_1, defaut9_1, optionConfig9_1, parent)
			grid.addWidget(self.spin9_1,9,0)
			if txt9_1: grid.addWidget(QLabel(txt9_1),9,1)
		if debut9_2:
			self.spin9_2 = SpinSlider(debut9_2, fin9_2, defaut9_2, optionConfig9_2, parent)
			grid.addWidget(self.spin9_2,9,2)
			if txt9_2: grid.addWidget(QLabel(txt9_2),9,3)
		if debut10_1:
			self.spin10_1 = SpinSlider(debut10_1, fin10_1, defaut10_1, optionConfig10_1, parent)
			grid.addWidget(self.spin10_1,10,0)
			if txt10_1: grid.addWidget(QLabel(txt10_1),10,1)
		if debut10_2:
			self.spin10_2 = SpinSlider(debut10_2, fin10_2, defaut10_2, optionConfig10_2, parent)
			grid.addWidget(self.spin10_2,10,2)
			if txt10_2: grid.addWidget(QLabel(txt10_2),10,3)
		if debut11_1:
			self.spin11_1 = SpinSlider(debut11_1, fin11_1, defaut11_1, optionConfig11_1, parent)
			grid.addWidget(self.spin11_1,11,0)
			if txt11_1: grid.addWidget(QLabel(txt11_1),11,1)
		if debut11_2:
			self.spin11_2 = SpinSlider(debut11_2, fin11_2, defaut11_2, optionConfig11_2, parent)
			grid.addWidget(self.spin11_2,11,2)
			if txt11_2: grid.addWidget(QLabel(txt11_2),11,3)
		if debut12_1:
			self.spin12_1 = SpinSlider(debut12_1, fin12_1, defaut12_1, optionConfig12_1, parent)
			grid.addWidget(self.spin12_1,12,0)
			if txt12_1: grid.addWidget(QLabel(txt12_1),12,1)
		if debut12_2:
			self.spin12_2 = SpinSlider(debut12_2, fin12_2, defaut12_2, optionConfig12_2, parent)
			grid.addWidget(self.spin12_2,12,2)
			if txt12_2: grid.addWidget(QLabel(txt12_2),12,3)
		if debut13_1:
			self.spin13_1 = SpinSlider(debut13_1, fin13_1, defaut13_1, optionConfig13_1, parent)
			grid.addWidget(self.spin13_1,13,0)
			if txt13_1: grid.addWidget(QLabel(txt13_1),13,1)
		if debut13_2:
			self.spin13_2 = SpinSlider(debut13_2, fin13_2, defaut13_2, optionConfig13_2, parent)
			grid.addWidget(self.spin13_2,13,2)
			if txt13_2: grid.addWidget(QLabel(txt13_2),13,3)
		if debut14_1:
			self.spin14_1 = SpinSlider(debut14_1, fin14_1, defaut14_1, optionConfig14_1, parent)
			grid.addWidget(self.spin14_1,14,0)
			if txt14_1: grid.addWidget(QLabel(txt14_1),14,1)
		if debut14_2:
			self.spin14_2 = SpinSlider(debut14_2, fin14_2, defaut14_2, optionConfig14_2, parent)
			grid.addWidget(self.spin14_2,14,2)
			if txt14_2: grid.addWidget(QLabel(txt14_2),14,3)
		if debut15_1:
			self.spin15_1 = SpinSlider(debut15_1, fin15_1, defaut15_1, optionConfig15_1, parent)
			grid.addWidget(self.spin15_1,15,0)
			if txt15_1: grid.addWidget(QLabel(txt15_1),15,1)
		if debut15_2:
			self.spin15_2 = SpinSlider(debut15_2, fin15_2, defaut15_2, optionConfig15_2, parent)
			grid.addWidget(self.spin15_2,15,2)
			if txt15_2: grid.addWidget(QLabel(txt15_2),15,3)
		if debut16_1:
			self.spin16_1 = SpinSlider(debut16_1, fin16_1, defaut16_1, optionConfig16_1, parent)
			grid.addWidget(self.spin16_1,16,0)
			if txt16_1: grid.addWidget(QLabel(txt16_1),16,1)
		if debut16_2:
			self.spin17_2 = SpinSlider(debut16_2, fin16_2, defaut16_2, optionConfig16_2, parent)
			grid.addWidget(self.spin16_2,16,2)
			if txt16_2: grid.addWidget(QLabel(txt16_2),16,3)
		if debut17_1:
			self.spin17_1 = SpinSlider(debut17_1, fin17_1, defaut17_1, optionConfig17_1, parent)
			grid.addWidget(self.spin17_1,17,0)
			if txt17_1: grid.addWidget(QLabel(txt17_1),17,1)
		if debut17_2:
			self.spin17_2 = SpinSlider(debut17_2, fin17_2, defaut17_2, optionConfig17_2, parent)
			grid.addWidget(self.spin17_2,17,2)
			if txt17_2: grid.addWidget(QLabel(txt17_2),17,3)
		if debut18_1:
			self.spin18_1 = SpinSlider(debut18_1, fin18_1, defaut18_1, optionConfig18_1, parent)
			grid.addWidget(self.spin18_1,18,0)
			if txt18_1: grid.addWidget(QLabel(txt18_1),18,1)
		if debut18_2:
			self.spin18_2 = SpinSlider(debut18_2, fin18_2, defaut18_2, optionConfig18_2, parent)
			grid.addWidget(self.spin18_2,18,2)
			if txt18_2: grid.addWidget(QLabel(txt18_2),18,3)
		if debut19_1:
			self.spin19_1 = SpinSlider(debut19_1, fin19_1, defaut19_1, optionConfig19_1, parent)
			grid.addWidget(self.spin19_1,19,0)
			if txt19_1: grid.addWidget(QLabel(txt19_1),19,1)
		if debut19_2:
			self.spin19_2 = SpinSlider(debut19_2, fin19_2, defaut19_2, optionConfig19_2, parent)
			grid.addWidget(self.spin19_2,19,2)
			if txt19_2: grid.addWidget(QLabel(txt19_2),19,3)
		if debut20_1:
			self.spin20_1 = SpinSlider(debut20_1, fin20_1, defaut20_1, optionConfig20_1, parent)
			grid.addWidget(self.spin20_1,20,0)
			if txt20_1: grid.addWidget(QLabel(txt20_1),20,1)
		if debut20_2:
			self.spin20_2 = SpinSlider(debut20_2, fin20_2, defaut20_2, optionConfig20_2, parent)
			grid.addWidget(self.spin20_2,20,2)
			if txt20_2: grid.addWidget(QLabel(txt20_2),20,3)
		grid.setAlignment(Qt.AlignCenter)
		grid.setMargin(0)
		
		'''
		if self.checkBoxTraitDebut.isChecked():
			try:
				self.spin1_1.setEnabled(True)
				self.spin2_1.setEnabled(True)
				self.spin3_1.setEnabled(True)
				self.spin4_1.setEnabled(True)
				self.spin5_1.setEnabled(True)
				self.spin6_1.setEnabled(True)
				self.spin7_1.setEnabled(True)
				self.spin8_1.setEnabled(True)
				self.spin9_1.setEnabled(True)
				self.spin10_1.setEnabled(True)
				self.spin11_1.setEnabled(True)
				self.spin12_1.setEnabled(True)
				self.spin13_1.setEnabled(True)
				self.spin14_1.setEnabled(True)
				self.spin15_1.setEnabled(True)
				self.spin16_1.setEnabled(True)
				self.spin17_1.setEnabled(True)
				self.spin18_1.setEnabled(True)
				self.spin19_1.setEnabled(True)
				self.spin20_1.setEnabled(True)

				self.spin1_2.setEnabled(False)
				self.spin2_2.setEnabled(False)
				self.spin3_2.setEnabled(False)
				self.spin4_2.setEnabled(False)
				self.spin5_2.setEnabled(False)
				self.spin6_2.setEnabled(False)
				self.spin7_2.setEnabled(False)
				self.spin8_2.setEnabled(False)
				self.spin9_2.setEnabled(False)
				self.spin10_2.setEnabled(False)
				self.spin11_2.setEnabled(False)
				self.spin12_2.setEnabled(False)
				self.spin13_2.setEnabled(False)
				self.spin14_2.setEnabled(False)
				self.spin15_2.setEnabled(False)
				self.spin16_2.setEnabled(False)
				self.spin17_2.setEnabled(False)
				self.spin18_2.setEnabled(False)
				self.spin19_2.setEnabled(False)
				self.spin20_2.setEnabled(False)
			except: pass
		'''
		
	def majChoixTraitementDebut(self):
		"Fonction pour collecte si choix du traitement début ou fin"
		
		if self.checkBoxTraitDebut.isChecked(): 
			# Réglages pour traitement fin est déselectionné
			self.checkBoxTraitFin.setChecked(False)
			self.checkBoxTraitDebut.setChecked(True)
			#
			self.checkTraitDebut = "trait_debut"
			return self.checkTraitDebut
			EkdPrint(u"Choix du traitement début")
			# L'exception empêche l'erreur:
			# Traceback (most recent call last):
			# ...
			# self.spin14_2.setEnabled(False)
			# AttributeError: 'SpinSliders20PossiblesDebutFin' object has no attribute 'spin14_2'
			'''
			try:
				self.spin1_2.setEnabled(False)
				self.spin2_2.setEnabled(False)
				self.spin3_2.setEnabled(False)
				self.spin4_2.setEnabled(False)
				self.spin5_2.setEnabled(False)
				self.spin6_2.setEnabled(False)
				self.spin7_2.setEnabled(False)
				self.spin8_2.setEnabled(False)
				self.spin9_2.setEnabled(False)
				self.spin10_2.setEnabled(False)
				self.spin11_2.setEnabled(False)
				self.spin12_2.setEnabled(False)
				self.spin13_2.setEnabled(False)
				self.spin14_2.setEnabled(False)
				self.spin15_2.setEnabled(False)
				self.spin16_2.setEnabled(False)
				self.spin17_2.setEnabled(False)
				self.spin18_2.setEnabled(False)
				self.spin19_2.setEnabled(False)
				self.spin20_2.setEnabled(False)
			 
				self.spin1_1.setEnabled(True)
				self.spin2_1.setEnabled(True)
				self.spin3_1.setEnabled(True)
				self.spin4_1.setEnabled(True)
				self.spin5_1.setEnabled(True)
				self.spin6_1.setEnabled(True)
				self.spin7_1.setEnabled(True)
				self.spin8_1.setEnabled(True)
				self.spin9_1.setEnabled(True)
				self.spin10_1.setEnabled(True)
				self.spin11_1.setEnabled(True)
				self.spin12_1.setEnabled(True)
				self.spin13_1.setEnabled(True)
				self.spin14_1.setEnabled(True)
				self.spin15_1.setEnabled(True)
				self.spin16_1.setEnabled(True)
				self.spin17_1.setEnabled(True)
				self.spin18_1.setEnabled(True)
				self.spin19_1.setEnabled(True)
				self.spin20_1.setEnabled(True)
			except: pass
			'''
			
		
	def majChoixTraitementFin(self):
		if self.checkBoxTraitFin.isChecked(): 
			# Réglages pour traitement fin est déselectionné
			self.checkBoxTraitDebut.setChecked(False)
			self.checkBoxTraitFin.setChecked(True)
			#
			self.checkTraitFin = "trait_fin"
			return self.checkTraitFin
			EkdPrint(u"Choix du traitement fin")
			# L'exception empêche l'erreur:
			# Traceback (most recent call last):
			# ...
			# self.spin14_2.setEnabled(True)
			# AttributeError: 'SpinSliders20PossiblesDebutFin' object has no attribute 'spin14_2'
			'''
			try:
				self.spin1_1.setEnabled(False)
				self.spin2_1.setEnabled(False)
				self.spin3_1.setEnabled(False)
				self.spin4_1.setEnabled(False)
				self.spin5_1.setEnabled(False)
				self.spin6_1.setEnabled(False)
				self.spin7_1.setEnabled(False)
				self.spin8_1.setEnabled(False)
				self.spin9_1.setEnabled(False)
				self.spin10_1.setEnabled(False)
				self.spin11_1.setEnabled(False)
				self.spin12_1.setEnabled(False)
				self.spin13_1.setEnabled(False)
				self.spin14_1.setEnabled(False)
				self.spin15_1.setEnabled(False)
				self.spin16_1.setEnabled(False)
				self.spin17_1.setEnabled(False)
				self.spin18_1.setEnabled(False)
				self.spin19_1.setEnabled(False)
				self.spin20_1.setEnabled(False)

				self.spin1_2.setEnabled(True)
				self.spin2_2.setEnabled(True)
				self.spin3_2.setEnabled(True)
				self.spin4_2.setEnabled(True)
				self.spin5_2.setEnabled(True)
				self.spin6_2.setEnabled(True)
				self.spin7_2.setEnabled(True)
				self.spin8_2.setEnabled(True)
				self.spin9_2.setEnabled(True)
				self.spin10_2.setEnabled(True)
				self.spin11_2.setEnabled(True)
				self.spin12_2.setEnabled(True)
				self.spin13_2.setEnabled(True)
				self.spin14_2.setEnabled(True)
				self.spin15_2.setEnabled(True)
				self.spin16_2.setEnabled(True)
				self.spin17_2.setEnabled(True)
				self.spin18_2.setEnabled(True)
				self.spin19_2.setEnabled(True)
				self.spin20_2.setEnabled(True)
			except: pass
			'''