# -*- coding: utf-8 -*-

import os, re
from PyQt4.QtCore import SIGNAL, QThread
from moteur_modules_common.EkdConfig import EkdConfig
from moteur_modules_common.EkdProcess import EkdProcess
from moteur_modules_common.EkdTools import debug
from gui_modules_animation.infoVideo import infovideo

# Détection durée vidéo avec certaines versions de FFmpeg avec pour un transcodage en WebM
from moteur_modules_common.FFmpegSpecialDureeVideoSec import SpecialDureeVideoSec

# Le calcul peut se faire en répartissant la charge de calcul sur les différents coeurs
from moteur_modules_common.FFmpegProspect import FFmpegCPU


class FFmpeg(QThread):
    """
    Classe permettant de ne gérer que la partie moteur de ffmpeg (sans avoir
    besoin d'interface graphique)
    """
    def __init__(self, inputfile, outputfile, codec=None, imgsize=None,
                 videosize=None, location=None):
        """
            codec : Identifiant du codec à utiliser
            inputfile : fichier audio/video d'entree
            outputfile : fichier de sortie
            imgsize : taille des images de sortie (conversion d'image en video)
            videosize : taille de la video en sortie
            location : chemin d'accèss à ffmpeg

            command  : ligne de commande du processus
            progress : progression du processus
            process  : processus ffmpeg de conversion
            log      : contenu des log de ffmpeg
        """
        super(FFmpeg, self).__init__()
        self.codec = codec
        self.inputfile = inputfile
        self.outputfile = outputfile

        if os.name in ['posix', 'mac'] and (location == None):
            #self.location=u"ffmpeg"
            # Utilisation de format pour une future compatibilité dans Python 3
            self.location=u"{0}ffmpeg".format(unicode(EkdConfig.urlFFmpeg)) 
        else :
            ## We are not on linux we force the ffmpeg location
            self.location=u"{0}ffmpeg.exe".format(EkdConfig.urlFFmpeg)

        self.dfltOptions = u""

        # Initialisation de la commande
        self.command = None

        # Initialisation de la progression
        self.progress = 0

        # Log d'output de la commande
        self.log = ""

        ### On passe par subprocess.popen
	
	# Appel de la classe pour le traitement multi-coeurs
	self.cl_nbre_cpu = FFmpegCPU()
	# Récup de nombre total de coeurs contenus dans la machine
	self.nbre_cpu = self.cl_nbre_cpu.nbre_cpu


    def setOutputFile(self, outputfile):
        self.outputfile = outputfile

    def setInputFile(self, inputfile):
        self.inputfile = inputfile

    def setVideoLen(self, videoLen):
        """ Définie la durée de la vidéo """
        self.videoLen = videoLen

    def setProgress(self, val):
        """
            Définie le niveau de progression et envoie un signal progress(int)
        """
        self.progress = val
        self.emit(SIGNAL("progress(int)"), self.progress)

    def addLog(self, line):
        """
            Ajout une ligne de log dans self.log et envoie un signal log(string)
        """
        try:
            #debug(u"FFmpeg log : %s" % line)
            self.log = u"{0}\n{1}".format(self.log, unicode(line, EkdConfig.coding))
            self.emit(SIGNAL("log(QString)"), line)
        except:
            #debug(u"FFmpeg log : %s" % line)
            self.log = u"{0}\n{1}".format(self.log, line)
            self.emit(SIGNAL("log(QString)"), line)

    def run(self):
        """ Démarre le processus ffmpeg """
        if self.command == None :
            debug(u"FFmpeg: Commande ffmpeg non définie")
            raise Exception
        self.addLog(self.command.encode(EkdConfig.coding))
        try :
            ## Attention, ffmpeg envoie sa sortie de progression sur stderr !!!
            self.process = EkdProcess( self.command.encode(EkdConfig.coding),
                                        stdinput = None, output = None)
            self.emit(SIGNAL("void started()"))
        except Exception, e:
            debug(u"Erreur dans le lancement de {0}\n({1})".format(self.command, e))
            raise
        self.retCode = None
        while self.retCode == None :
            self.progression()
            self.retCode = self.process.poll()
            self.usleep(10)
        self.end(self.retCode)

    def cancel(self):
        """ Cancel the process if it is currently running """
        try:
            self.process.terminate()
        except:
            pass

    def end(self, returnCode):
        """ Traite la fin du processus ffmpeg """
        #if returnCode == QProcess.CrashExit:
        if returnCode != 0:
            self.addLog(u"Une erreur est survenue"
                        "lors de l'encodage".encode(EkdConfig.coding))
        self.emit(SIGNAL("finished(int)"), returnCode)

    def progression(self):
        """
        Recupération de la progression du process en cours
        """
	'''
	try:
	    if self.videoLen==0 : self.videoLen = 1
	except: pass
        line = self.process.stderr.readline()
        if re.match('^frame=.*', line):
            val = re.findall("time=([^ ]+)",line)[0]
	    # --------------------------------------------------------------- #
	    # Détection durée vidéo avec certaines versions de FFmpeg 
	    # pour un transcodage en WebM
	    # --------------------------------------------------------------- #
	    # Si la valeur de val est du style: 00:00:47.01 ... la valeur
	    # est recalculée ...
            if ':' in val:
		# Utilisation de la classe SpecialDureeVideoSec
		tempSec = SpecialDureeVideoSec(val)
		# Appel de la fonction calculDureeSec de la classe 
		# SpecialDureeVideoSec
		val = tempSec.calculDureeSec()
	    # Autrement si la valeur de val est du style: 47.01, c'est à dire
	    # la valeur du nombre totale de secondes contenues dans la vidéo, 
	    # la valeur est utilisée telle quelle ...
	    else: pass
            # --------------------------------------------------------------- #
            self.setProgress(int( ( float(val) * 100 / self.videoLen) ))
        else:
            self.addLog(line)
	'''
	
	#################################################
	# Changement le 28/10/12. Certains traitements ne se font pas (et cela génère une
	# erreur comme quoi self.videoLen n'est pas trouvé). Des exceptions ont été placées
	# au niveau de : if self.videoLen==0 ... et : if re.match('^frame=.*', line): ...
	# self.addLog(line) ... comme cela la détection de la durée de la vidéo n'est pas 
	# convoquée (certaines vidéos ne la contiennent pas). Avec certains traitements 
	# (peu nombreux), la barre de progression passe de 0 à 100 (et reste bloquée à 100
	# le temps du traitement de la/des vidéo(s))), mais c'est un moindre mal ... le calcul
	# se fera quand même.
	#################################################
	
	try:
	    if self.videoLen==0 : self.videoLen = 1
	except: pass
        line = self.process.stderr.readline()
	try:
            if re.match('^frame=.*', line):
                val = re.findall("time=([^ ]+)",line)[0]
	        # --------------------------------------------------------------- #
	        # Détection durée vidéo avec certaines versions de FFmpeg 
	        # pour un transcodage en WebM
	        # --------------------------------------------------------------- #
	        # Si la valeur de val est du style: 00:00:47.01 ... la valeur
	        # est recalculée ...
                if ':' in val:
		    # Utilisation de la classe SpecialDureeVideoSec
		    tempSec = SpecialDureeVideoSec(val)
		    # Appel de la fonction calculDureeSec de la classe 
		    # SpecialDureeVideoSec
		    val = tempSec.calculDureeSec()
	        # Autrement si la valeur de val est du style: 47.01, c'est à dire
	        # la valeur du nombre totale de secondes contenues dans la vidéo, 
	        # la valeur est utilisée telle quelle ...
	        else: pass
                # --------------------------------------------------------------- #
                self.setProgress(int( ( float(val) * 100 / self.videoLen) ))
            else: self.addLog(line)
	except:
	    self.setProgress(90)
	    self.addLog(line)

    def setCommandVideo(self, size, vcodec, bitrate, aspect, acodec):
        """
            Définie la ligne de commande de façon générique pour le traitement
            video
        """
        self.command = u"{0} {1} -i \"{2}\" -vcodec {3} -b:v {4} -vf yadif -s {5} -aspect {6} -acodec {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, vcodec, bitrate, size, aspect, acodec, self.nbre_cpu, self.outputfile)

    def setCommandAudio(self, vbitrate="44100", abitrate="192k", achannel="2", format="wav"):
        """ Définie la ligne de commande de façon générique pour le traitement audio """
        self.command = u"{0} {1} -i \"{2}\" -vn -ar {3} -ab {4} -ac {5} -f {6} -threads {7} -y \"{8}\"".format(self.location, self.dfltOptions, self.inputfile, vbitrate, abitrate, achannel, format, self.nbre_cpu, self.outputfile)

    def commandeFfmpegConvPourLeWeb(self, syntaxeFFmpeg):
	""" Défini la ligne de commande pour les différentes spécifications de transcodage pour Youtube et pour Dailymotion """
	# * Youtube: http://support.google.com/youtube/bin/static.py?hl=fr&guide=1728585&topic=1728573&page=guide.cs
	# Dailymotion: http://www.dailymotion.com/upload/faq
	# Vimeo: http://vimeo.com/help/compression
	self.command = u"{0} {1} -i \"{2}\" {3} -vf yadif -threads {4} -y \"{5}\"".format(self.location, self.dfltOptions, self.inputfile, syntaxeFFmpeg, self.nbre_cpu, self.outputfile)

    def commandeFfmpegConvTHD(self, size, aspect, vcodec="mpeg4", vbitrate="6000k", acodec="pcm_s16be"):
        """ Définie la ligne de commande pour la conversion en Très Haute Définition (4K, 2.7K, 2K, ...) """
        self.command = u"{0} {1} -i \"{2}\" -vcodec {3} -b:v {4} -vf yadif -s {5} -aspect {6} -acodec {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, vcodec, vbitrate, size, aspect, acodec, self.nbre_cpu, self.outputfile)

    def commandeFfmpegConvHD(self, size, vcodec="mpeg4", vbitrate="6000k", aspect="16:9", acodec="pcm_s16be"):
        """ Définie la ligne de commande pour la conversion HD """
        self.command = u"{0} {1} -i \"{2}\" -vcodec {3} -b:v {4} -vf yadif -s {5} -aspect {6} -acodec {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, vcodec, vbitrate, size, aspect, acodec, self.nbre_cpu, self.outputfile)

    def commandeFfmpegConvHD_DNxHD(self, size, vcodec, rate, vbitrate, acodec="copy"):
        """ Définie la ligne de commande pour la conversion HD pour les différents 
	traitement avec le codec Avid DNxHD """
	# Commande (exemple): ffmpeg -i video_in.avi -s 1280x720 -r 23.976 -b 90000k -vcodec dnxhd -deinterlace -acodec copy -y video_out.mov
        self.command = u"{0} {1} -i \"{2}\" -s {3} -r {4} -b:v {5} -vcodec {6} -vf yadif -acodec {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, size, rate, vbitrate, vcodec, acodec, self.nbre_cpu, self.outputfile)

    def commandeFfmpegSeparation(self, vbitrate="44100", achannel="2", format="wav"):
        """ Définie la commande de séparation de l'audio et de la vidéo """
        self.setCommandAudio(vbitrate=vbitrate, achannel=achannel, format=format)

    def commandeFfmpegEncodageWav(self, vbitrate="44100", abitrate="192k", achannel="2", format="wav"):
        """ Définie la commande d'encodage Wav """
        self.setCommandAudio(vbitrate, abitrate, achannel, format)

    def commandeFfmpegNbrImgSec(self, rate, sync="1", vbitrate="800k"):
        """ Définie la commande de modification du nombre d'image par seconde """
        self.command = u"{0} {1} -i \"{2}\" -r {3} -vsync {4} -b:v {5} -threads {6} -y \"{7}\"".format(self.location, self.dfltOptions, self.inputfile, rate, sync, vbitrate, self.nbre_cpu, self.outputfile)

    def commandeFfmpegConvAnimImg(self):
        """ Définie la commande pour la conversion d'une video en image """
        self.setOutputFile(unicode(self.outputfile) + u"_%09d.png")
        self.command = u"{0} {1} -i \"{2}\" -threads {3} -y \"{4}\"".format(self.location, self.dfltOptions, self.inputfile, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageDv(self, format="dv", target="pal-dv"):
        """ Définie la commande pour la conversion d'une vidéo au format DV"""
        self.command = u"{0} {1} -i \"{2}\" -f {3} -target {4} -threads {5} -y \"{6}\"".format(self.location, self.dfltOptions, self.inputfile, format, target, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageMov(self, comp, size, desent, vcodec="mov", acodec="pcm_s16be"):
        """ Définie la commande pour l'encodage en format mov """
        # --> La valeur de la compression (pour le MOV) se fait maintenant dans 
        # animation_encodage_general.py (fonction appliquer --> elif j[0] in ['codec_mov_ffmpeg']:)
	self.command = u"{0} {1} -i \"{2}\" {3} {4} {5} -f {6} -acodec {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, comp, size, desent, vcodec, acodec, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageSWF(self, comp, size, desent, vcodec="swf"):
        """ Définie la commande pour l'encodage en format ShockWave Flash"""
	self.command = u"{0} {1} -i \"{2}\" {3} {4} -f {5} -ar 44100 {6} -threads {7} -y \"{8}\"".format(self.location, self.dfltOptions, self.inputfile, comp, size, vcodec, desent, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageTS(self, comp, size, desent, vcodec="mpegts"):
        """ Définie la commande pour l'encodage en format ShockWave Flash"""
	self.command = u"{0} {1} -i \"{2}\" {3} {4} -f {5} {6} -threads {7} -y \"{8}\"".format(self.location, self.dfltOptions, self.inputfile, comp, size, vcodec, desent, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageHfyu(self, size, desent, audio=True, vcodec="huffyuv", acodec="pcm_s16be", format="yuv422p"):
        """ Définie la commande d'encodage Hfyu """
        if audio:
            self.command = u"{0} {1} -i \"{2}\" {3} -vcodec {4} -pix_fmt {5} {6} -qscale 0 -threads {7} -y \"{8}\"".format(self.location, self.dfltOptions, self.inputfile, size, vcodec, format, desent, self.nbre_cpu, self.outputfile)
        else:
            self.command = u"{0} {1} -i \"{2}\" {3} -vcodec {4} -pix_fmt {5} {6} -qscale 0 -acodec {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, size, vcodec, format, desent, acodec, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageVob(self, vquantizer, audiofreq="44100"):
        """ Définie la commande d'encodage en vob """
        # --> La valeur de la compression (pour le VOB) se fait maintenant dans 
        # animation_encodage_general.py (fonction appliquer --> elif j[0] in ['codec_mov_ffmpeg']:)
        self.command = u"{0} {1} -i \"{2}\" -ar {3} {4} -threads {5} -y \"{6}\"".format(self.location, self.dfltOptions, self.inputfile, audiofreq, vquantizer, self.nbre_cpu, self.outputfile)

    def commandeFfmpegEncodageWebM(self, size, vbitrate, desent):
        """ Définie la commande pour l'encodage en format webm"""
	# Ancienne syntaxe pour le WebM
        #self.command = u"{0} {1} -i \"{2}\" {3} {4} -threads {5} -y \"{6}\"".format(self.location, self.dfltOptions, self.inputfile, size, desent, self.nbre_cpu, self.outputfile)
	# Nouvelle syntaxe pour le WebM, à partir de celle trouvée ici: https://www.virag.si/2012/01/webm-web-video-encoding-tutorial-with-ffmpeg-0-9
        self.command = u"{0} {1} -i \"{2}\" {3} -codec:v libvpx -quality good -cpu-used 0 -b:v {4}k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -codec:a libvorbis -b:a 128k -f webm {5} -threads {6} -y \"{7}\"".format(self.location, self.dfltOptions, self.inputfile, size, vbitrate, desent, self.nbre_cpu, self.outputfile)
    
    def commandeFfmpegEncodageProRes(self, size, desent):
        """ Définie la commande pour l'encodage en format Apple ProRes 422"""
        self.command = u"{0} {1} -i \"{2}\" -codec:v prores -profile:v 3 -codec:a pcm_s16le {3} {4} -threads {5} -y \"{6}\"".format(self.location, self.dfltOptions, self.inputfile, size, desent, self.nbre_cpu, self.outputfile)
    
    def commandeFfmpegEncodageH264Mpeg4(self, vbitrate, size, desent):
        """ Définie la commande pour l'encodage en format h264 mpeg4 """
	self.command = u"{0} {1} -i \"{2}\" -f mp4 -c:v libx264 -b:v {3} {4} {5} -threads {6} -y \"{7}\"".format(self.location, self.dfltOptions, self.inputfile, vbitrate, size, desent, self.nbre_cpu, self.outputfile)
    
    def commandeFfmpegEncodageMkv(self, format_video, format_audio, comp, size, desent):
        """ Définie la commande pour l'encodage en format mkv """
        self.command = u"{0} {1} -i \"{2}\" {3} {4} {5} {6} {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, self.inputfile, format_video, format_audio, comp, size, desent, self.nbre_cpu, self.outputfile)
    
    def commandeFfmpegEncodageDVD(self, mode_dvd, apparence_dvd):
	""" Définie la commande pour l'encodage en format DVD """
        self.command = u"{0} {1} -i \"{2}\" {3} {4} -threads {5} -y \"{6}\"".format(self.location, self.dfltOptions, self.inputfile, mode_dvd, apparence_dvd, self.nbre_cpu, self.outputfile)
    
    def commandeFfmpegEncodageAMV(self, size, desent, achannel="1", audiofreq="22050", qmin="3", qmax="3"):
        """ Définie la commande d'encodage en AMV """
        self.command = u"{0} {1} -i \"{2}\" -s {3} -ac {4} -ar {5} -qmin {6} -qmax {7} {8} -threads {9} -y \"{10}\"".format(self.location, self.dfltOptions, self.inputfile, size, achannel, audiofreq, qmin, qmax, desent, self.nbre_cpu, self.outputfile)
        # La commande: ffmpeg -i entree.avi -s 160x120 -ac 1 -ar 22050 -qmin 3 -qmax 3 sortie.avi
    
    '''
    def commandeFfmpegEncodage3gp(self, vcodec="3gp", size="qcif", acodec="aac", audio=True):
        """ Définie la commande d'encodage en 3gp """
        if audio :
            self.command = u"%s %s -i \"%s\" -f %s -sameq -s %s -acodec %s -y \"%s\"" % \
                    (self.location, self.dfltOptions, self.inputfile, vcodec, size, acodec, self.outputfile)
        else:
            self.command = u"%s %s -i \"%s\" -f %s -sameq -s %s -y \"%s\"" % \
                    (self.location, self.dfltOptions, self.inputfile, vcodec, size, self.outputfile)
    '''

    # Pour l'instant le son n'est pas géré
    def commandeFfmpegEncodage3gp(self, size, desent, vcodec="3gp", acodec="aac", audio=True):
        """ Définie la commande d'encodage en 3gp """
	self.command = u"{0} {1} -i \"{2}\" -s {3} -f {4} {5} -qscale 0 -threads {6} -y \"{7}\"".format(self.location, self.dfltOptions, self.inputfile, size, vcodec, desent, self.nbre_cpu, self.outputfile)
    
    # Filtre vidéo géré par FFmpeg
    def commandeFiltreVideoDesentrelacer(self): 
        self.command = u"{0} {1} -i \"{2}\" -vf yadif -qscale 0 -threads {3} -y \"{4}\"".format(self.location, self.dfltOptions, self.inputfile, self.nbre_cpu, self.outputfile)
    
    '''
    # Filtre vidéo géré par FFmpeg
    def commandeFiltreVideoTimeLapse(self, acceleration): 
        self.command = u"{0} {1} -i \"{2}\" -vf \"setpts={3:.2f}*PTS,yadif\" -an -sameq -y \"{4}\"".format(self.location, self.dfltOptions, self.inputfile, acceleration, self.outputfile)
    '''
    
    # Filtre vidéo géré par FFmpeg
    def commandeFiltreVideoSlowMotion(self, ralenti): 
        self.command = u"{0} {1} -i \"{2}\" -vf \"setpts={3:.2f}*PTS,yadif\" -an -qscale 0 -threads {4} -y \"{5}\"".format(self.location, self.dfltOptions, self.inputfile, ralenti, self.nbre_cpu, self.outputfile)
	
    # Filtre vidéo géré par FFmpeg
    def commandeFiltreVideoContourBlancFondNoir(self): 
        self.command = u"{0} {1} -i \"{2}\" -vf edgedetect -qscale 0 -threads {3} -y \"{4}\"".format(self.location, self.dfltOptions, self.inputfile, self.nbre_cpu, self.outputfile)
	
    # Filtre vidéo géré par FFmpeg
    def commandeFiltreVideoStabilisateur(self):
	self.command = u"{0} {1} -i \"{2}\" -vf deshake -qscale 0 -threads {3} -y \"{4}\"".format(self.location, self.dfltOptions, self.inputfile, self.nbre_cpu, self.outputfile)
	
    # Filtre vidéo géré par FFmpeg
    def commandeFiltreVideoDebruitageVideo(self):
	self.command = u"{0} {1} -i \"{2}\" -vf hqdn3d -qscale 0 -threads {3} -y \"{4}\"".format(self.location, self.dfltOptions, self.inputfile, self.nbre_cpu, self.outputfile)

    # Filtre vidéo géré par FFmpeg
    def commandeFiltreBandesExterieuresPredef(self, taille_vid_w, taille_vid_h, video_aspect, bgauche, bdroite, bbas, bhaut, coul_hex):
	# Dimensions des vidéos chargées (en entrée)
	info = infovideo(self.inputfile)
	id_video_largeur = info.videoLargeur
        id_video_hauteur = info.videoHauteur
        # Dimensions des vidéos en sortie
        tv_vid_sortie_w = taille_vid_w
        tv_vid_sortie_h = taille_vid_h
	# Pour contrôler le calcul (dim vidéos chargées par l'utilisateur)
	id_tv = str(taille_vid_w)+'x'+str(taille_vid_h)
	# Calcul du ratio des vidéos chargées
	ratio_vid_chargee = float(id_video_largeur) / float(id_video_hauteur)
	
	# Si preset de chargement avec vidéos 16/9, 16/10 ou ratio supérieur ou égal à 1.5
	if id_tv in ['3840x2160', '1920x1080', '1280x720', '960x540', '852x480', '1024x576', '720x480', '800x480']:
	    # ----------------------------------------------------------------
	    # FFmpeg ne peut pas travailler avec des vidéos dont les dimensions sont impaires
	    if id_video_largeur % 2 == 1: taille_vid_w = int(id_video_largeur) - 1
	    if id_video_hauteur % 2 == 1: taille_vid_h = int(id_video_hauteur) - 1
	    # Calcul de la future largeur de la vidéo en sortie
	    larg_vid_calc = float(taille_vid_h) * ratio_vid_chargee
	    # Calcul des dimensions de la bande destinée aux côtés gauche et droit
	    larg_band = float(taille_vid_w) - larg_vid_calc
	    # Calcul des dimensions de chaque bande (gauche et droite)
	    larg_chaque_bande = larg_band / 2.0
	    # Taille redimensionnement vidéo sans les bandes
	    taille_vid_w = int(larg_vid_calc)
	    # Si le calcul donne une taille impaire, la vidéo est redimensionnée
	    if int(taille_vid_w) % 2 == 1: taille_vid_w = int(taille_vid_w) - 1
	    # Largeur de chaque bande
	    bgauche = int(larg_chaque_bande)
	    bdroite = int(larg_chaque_bande)
	    # Les valeurs impaires ne sont pas acceptées par FFmpeg
	    # --> si la dimension des bandes donne un résultat impair
	    # ces bandes doivent être redimensionnées
	    if bgauche % 2 == 1: bgauche = bgauche - 1
	    if bdroite % 2 == 1: bdroite = bdroite - 1
	    # ----------------------------------------------------------------
	    # Dimension actuelle en largeur plus dimension des des
	    # bandes gauche et droite (après retrait d'un pixel)
	    dim_tot_vid_plus_band = taille_vid_w + bgauche + bdroite
	    # Si cette dimension actuelle en largeur est inférieure
	    # à la dimension en largeur sélectionnée par l'utilisateur
	    # dans la liste déroulante Taille de la vidéo en sortie
	    if dim_tot_vid_plus_band < tv_vid_sortie_w:
		# Calcul des pixels résiduels
		calc_v_sort_v_ent = int(tv_vid_sortie_w) - dim_tot_vid_plus_band
		cal_dim_chaq_band = calc_v_sort_v_ent / 2
		# Attribution des nouvelles dimensions des bandes
		bgauche, bdroite = bgauche + cal_dim_chaq_band, bdroite + cal_dim_chaq_band
		if bgauche % 2 == 1: bgauche = bgauche + 1
		if bdroite % 2 == 1: bdroite = bdroite + 1
 
	    # ------------------ #
	    # Traitement final
	    # ------------------ #
	    tail_w_vid_source = tv_vid_sortie_w
	    tail_h_vid_source = tv_vid_sortie_h
	    bande_x = bgauche
	    bande_y = 0
	    # (0x-->+valeur hexadécimale de la couleur)
	    # Exemple commande ffmpeg: ... -vf "scale=900:720:interl=1,pad=1280:720:190:0:0x83b7ff,setdar=16:9" ...
	    self.command = u"{0} {1} -i \"{2}\" -vf \"scale={3}:{4}:interl=1,pad={5}:{6}:{7}:{8}:0x{9},setdar={10}\" -qscale 0 -threads {11} -y \"{12}\"".format(self.location, self.dfltOptions, self.inputfile, taille_vid_w, taille_vid_h, tail_w_vid_source, tail_h_vid_source, bande_x, bande_y, coul_hex, video_aspect,self.nbre_cpu,  self.outputfile)

	# Pour le reste c'est à dire 4/3 ou ratios < 1.5    
	else:
	    # ----------------------------------------------------------------
	    # FFmpeg ne peut pas travailler avec des vidéos dont les dimensions sont impaires
	    if id_video_largeur % 2 == 1: taille_vid_w = int(id_video_largeur) - 1
	    if id_video_hauteur % 2 == 1: taille_vid_h = int(id_video_hauteur) - 1
	    # Calcul de la future hauteur de la vidéo en sortie
	    haut_vid_calc = float(taille_vid_w) / ratio_vid_chargee
	    # Calcul des dimensions de la bande destinée aux côtés bas et haut
	    haut_band = float(taille_vid_h) - haut_vid_calc
	    # Calcul des dimensions de chaque bande (bas et haut)
	    haut_chaque_bande = haut_band / 2.0
	    # Taille redimensionnement vidéo sans les bandes
	    taille_vid_h = int(haut_vid_calc)
	    # Si le calcul donne une taille impaire, la vidéo est redimensionnée
	    if int(taille_vid_h) % 2 == 1: taille_vid_h = int(taille_vid_h) - 1
	    # Largeur de chaque bande
	    bbas = int(haut_chaque_bande) 
	    bhaut = int(haut_chaque_bande)
	    # Les valeurs impaires ne sont pas acceptées par FFmpeg
	    # --> si la dimension des bandes donne un résultat impair
	    # ces bandes doivent être redimensionnées
	    if bbas % 2 == 1: bbas = bbas - 1
	    if bhaut % 2 == 1: bhaut = bhaut - 1
	    # ----------------------------------------------------------------
	    # Dimension actuelle en hauteur plus dimension des 
	    # des bandes bas et haut (après retrait d'un pixel)
	    dim_tot_vid_plus_band = taille_vid_h + bbas + bhaut
	    # Si cette dimension actuelle en hauteur est inférieure
	    # à la dimension en hauteur sélectionnée par l'utilisateur
	    # dans la liste déroulante Taille de la vidéo en sortie
	    if dim_tot_vid_plus_band < tv_vid_sortie_h: 
		# Calcul des pixels résiduels
		calc_v_sort_v_ent = int(tv_vid_sortie_h) - dim_tot_vid_plus_band
		cal_dim_chaq_band = calc_v_sort_v_ent / 2
		# Attribution des nouvelles dimensions des bandes
		bbas, bhaut = bbas + cal_dim_chaq_band, bhaut + cal_dim_chaq_band
		if bbas % 2 == 1: bbas = bbas + 1
		if bhaut % 2 == 1: bhaut = bhaut + 1

	    # ------------------ #
	    # Traitement final
	    # ------------------ #
	    # Valeurs spécifiques
	    tail_w_vid_source = tv_vid_sortie_w
	    tail_h_vid_source = tv_vid_sortie_h
	    bande_x = 0
	    bande_y = bbas
	    # (0x-->+valeur hexadécimale de la couleur)
	    # Exemple commnde ffmpeg: ... -vf "scale=768:432:interl=1,pad=768:576:0:72:0x808080,setdar=4:3" ...
            self.command = u"{0} {1} -i \"{2}\" -vf \"scale={3}:{4}:interl=1,pad={5}:{6}:{7}:{8}:0x{9},setdar={10}\" -qscale 0 -threads {11} -y \"{12}\"".format(self.location, self.dfltOptions, self.inputfile, taille_vid_w, taille_vid_h, tail_w_vid_source, tail_h_vid_source, bande_x, bande_y, coul_hex, video_aspect, self.nbre_cpu, self.outputfile)
    
    def commandeFfmpegConvertImg(self, rate, size, aspect, vcodec, format="image2", qscale="1"):
        """ Définie la commande de convertion d'images en video """
        if vcodec == 'dv' :
            vcodec = '-target pal-dv'
            format = "-f dv"
        elif vcodec == 'mov':
            format = '-f mov'
            vcodec = ''
        elif vcodec == 'vob':
            vcodec = ''
            format = ""
            rate = ""
        elif vcodec == 'mp4':
            vcodec = ""
            format = "-f mp4"
        elif vcodec == 'pro_res':
            vcodec = "-codec:v prores -profile:v 3"
            format = ""
	elif vcodec == 'avi_raw':
            vcodec = "-codec:v copy -pix_fmt yuv420p"
            format = "-f avi"
	elif vcodec == "h264_mp4":
            vcodec = "-c:v libx264"
            format = "-f mp4"
	elif vcodec == "ts":
            vcodec = ""
            format = "-f mpegts"
	elif vcodec == "skockwave_flash":
            vcodec = ""
            format = "-f swf"
        elif vcodec == 'webm':
            vcodec = "-codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k"
            format = "-f webm"
        elif vcodec == 'wmv':
            vcodec = ""
            format = ""
        elif vcodec == 'ljpeg':
            vcodec = "-vcodec {0}".format(vcodec)
            self.dfltOptions = ''
            ## On ajoute le format yuv420p qui permet une conversion directe en
            ## ljpeg contrairement au format par défaut
            format = "-pix_fmt yuvj420p"
        else :
            vcodec = "-vcodec {0}".format(vcodec)
            format = ""
            self.dfltOptions = ''
        if rate != "" :
	    rate = "-r {0}".format(rate)

	# vcodec et aspect avant format, et pas de syntaxe: ... -q:v ... (pour FFmpeg)
	if self.codec in ["pro_res", "webm", "h264_mp4", "ts", "skockwave_flash"]:
		self.command = u"{0} {1} {2} -s {3} {4} -f image2 -i \"{5}\" {6} {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, rate, size, aspect, self.inputfile, vcodec, format, self.nbre_cpu, self.outputfile)
	# ajout de: -f rawvideo format avant vcodec et pas de syntaxe: ... -q:v ... (pour FFmpeg) ... et ajout de aspect
	elif self.codec in ['avi_raw']:
		self.command = u"{0} {1} -f rawvideo {2} -s {3} {4} -f image2 -i \"{5}\" {6} {7} -threads {8} -y \"{9}\"".format(self.location, self.dfltOptions, rate, size, aspect, self.inputfile, format, vcodec, self.nbre_cpu, self.outputfile)
	else:
		self.command = u"{0} {1} {2} -s {3} {4} -f image2 -i \"{5}\" {6} {7} -q:v {8} -threads {9} -y \"{10}\"".format(self.location, self.dfltOptions, rate, size, aspect, self.inputfile, format, vcodec, qscale, self.nbre_cpu, self.outputfile)
	
    def commandeFfmpegConv_16_9_ou_4_3(self, ext, size, aspect, audio=True):
        """ Définie la commande de convertion de résolution 16/9 4/3 """

        a_codec_d_s = '-vf yadif'
	aspect = str(aspect)
        if ext == '.avi':
            v_codec = '-vcodec mjpeg'
            self.addLog(_(u"Vous avez décidé de traiter une vidéo AVI, sachez qu'elle sera encodée (votre vidéo) en Motion JPEG (extension .avi).\n"))
        elif ext == '.mov':
            v_codec = '-vcodec mpeg4'
            a_codec_d_s = '-acodec pcm_s16be {0}'.format(a_codec_d_s)
        elif ext == '.vob':
            v_codec = ''
        elif ext == '.mp4':
            v_codec = '-f mp4 -c:v libx264'
            if audio:
		a_codec_d_s = u'-acodec copy {0}'.format(a_codec_d_s)
        elif ext == '.ts':
            v_codec = '-f mpegts'
            if audio:
		a_codec_d_s = u'-acodec copy {0}'.format(a_codec_d_s)
        elif ext == '.webm':
            v_codec = '-codec:v libvpx -quality good -cpu-used 0 -b:v 1200k -qmin 10 -qmax 42 -maxrate 500k -bufsize 1000k -f webm'
	    a_codec_d_s = '-codec:a libvorbis -b:a 128k {0}'.format(a_codec_d_s)
        elif ext == '.flv':
            v_codec = u'-f flv'
            if audio:
		a_codec_d_s = u'-acodec copy {0}'.format(a_codec_d_s)
        else: #'.mpg', '.mpeg'
            v_codec = '-vcodec mpeg2video'
            self.addLog(_(u"Vous avez décidé de traiter une vidéo MPEG, sachez qu'elle sera encodée (votre vidéo) MPEG2 (extension .mpg ou .mpeg).\n"))

        self.command = u"{0} -i \"{1}\" {2} {3} -s {4} -aspect {5} -qscale 0 -threads {6} -y \"{7}\"".format(self.location, self.inputfile, v_codec, a_codec_d_s, size, aspect, self.nbre_cpu, self.outputfile)

    def commandeConcatAudio(self):
            self.command = u"{0} -i \"{1}\" -ab 192k -ar 44100 -ac 2 -f wav -threads {2} -y \"{3}\"".format(self.location, self.inputfile, self.nbre_cpu, self.output)

    def commandeConcatAudioFilterComplex(self, fich_entree_a, nbre_audio_concat):
	    """ Concaténation des fichiers audio pour le montage vidéo (Vidéo + audio). Utilisation du filtre filter_complex de FFmpeg """
            self.command = u"{0} {1} -filter_complex concat=n={2}:v=0:a=1 -f wav -vn -threads {3} -y \"{4}\"".format(self.location, fich_entree_a, nbre_audio_concat, self.nbre_cpu, self.outputfile)
    
    def commandeConcatVideoFilterComplex(self, fich_entree_v, nbre_video_concat):
            """ Concaténation des vidéos pour le montage vidéo (Vidéo + audio). Utilisation du filtre filter_complex de FFmpeg """
	    self.command = u"{0} {1} -filter_complex concat=n={2}:v=1:a=0 -codec:v mjpeg -b:v 5000k -an -threads {3} -y \"{4}\"".format(self.location, fich_entree_v, nbre_video_concat, self.nbre_cpu, self.outputfile)
	
    def commandeConcatVideoSeulAvecSonFilterComplex(self, fich_entree_v, nbre_video_concat, audio, synchro_audio):
            """ Concaténation des vidéos pour le montage vidéo (Vidéo seulement). Utilisation du filtre filter_complex de FFmpeg """
	    # Utilisation de la variable audio pour détection si les vidéos ne possèdent pas de canal audio : si canal audio 
	    # présent dans toutes les vidéos a=1, autrement a=0 (et ce dans la commande FFmpeg)
	    self.command = u"{0} {1} -filter_complex concat=n={2}:v=1:a={3} -codec:v mjpeg -b:v 5000k {4} -threads {5} -y \"{6}\"".format(self.location, fich_entree_v, nbre_video_concat, audio, synchro_audio, self.nbre_cpu, self.outputfile)
