# -*- coding: utf-8 -*-

import os, re
from PyQt4.QtCore import SIGNAL, QString, QThread
from moteur_modules_common.EkdConfig import EkdConfig
from moteur_modules_common.EkdProcess import EkdProcess
from moteur_modules_common.EkdTools import debug
# Le calcul peut se faire en répartissant la charge de calcul sur les différents coeurs
from moteur_modules_common.FFmpegProspect import FFmpegCPU


# --> Ceci a été fait pour séparer les traitements gérés par Mencoder de ceux gérés par
# FFmpeg et FFmpeg2theora
######################################################################################
### Ici il s'agit de l'extraction d'image pour la prévisualisation du résultat
### de l'effet pour les filtres vidéo. C'est Mencoder qui s'occupe des filtres vidéo
### ... MAIS ... pour cette partie spécifique d'extraction des images c'est FFmpeg
### qui est utilisé !.

# La syntaxe Mplayer a été remplacée par la syntaxe FFmpeg car la syntaxe
# Mplayer generait des bugs dans l'extraction des images --> avec certaines
# vidéos (et de façon alléatoire). Le rendu est un peu plus lent qu'avec
# Mplayer mais plus de bugs.
def extraireImage(cheminVideoSource, cheminRepDestination, valeurNum, multiple=True):
    "Extraction d'une image d'une vidéo"

    patern = ""
    if multiple :
        patern = u"_%8d.jpg"
	
    if os.name == 'nt' :
        urlFFmpeg = u'{0}ffmpeg.exe'.format(EkdConfig.urlFFmpeg)
    else :
        urlFFmpeg = u'ffmpeg'
		
    commande = u"{0} -i \"{1}\" -vframes 1 -ss {2} -an -f image2 \"{3}\"".format(urlFFmpeg, cheminVideoSource, str(valeurNum), cheminRepDestination+patern)
    p=EkdProcess(commande.encode(EkdConfig.coding))
    p.wait()
######################################################################################

if os.name in ['posix', 'mac']:
    unix = True
    coding = "utf-8"
else :
    unix = False
    coding = "cp1252"

class Mencoder(QThread):
    """
    Classe permettant de ne gérer que la partie moteur de mencoder (sans avoir
    besoin d'interface graphique)
    """
    def __init__(self, inputfile, outputfile, codec=None, imgsize=None,
                 videosize=None, location=None):
        """
            codec : Identifiant du codec à utiliser
            inputfile : fichier audio/video d'entree
            outputfile : fichier de sortie
            imgsize : taille des images de sortie (conversion d'image en video)
            videosize : taille de la video en sortie
            location : chemin d'accèss à mencoder

            command  : ligne de commande du processus
            progress : progression du processus
            process  : processus mencoder de conversion
            log      : contenu des log de mencoder
        """
        super(Mencoder, self).__init__()
        self.codec = codec

        self.setInputFile(inputfile)
        self.outputfile = outputfile

        if unix:# and (location == None):
            self.location=u"mencoder"
        else :
            ## We are not on linux we force the mencoder location
            self.location=u"{0}mencoder.exe".format(EkdConfig.urlMencoder)

        self.dfltOptions = ""

        # Initialisation de la commande
        self.command = None

        # Initialisation de la progression
        self.progress = 0

        # Log d'output de la commande
        self.log = ""
	
	# Appel de la classe pour le traitement multi-coeurs
	self.cl_nbre_cpu = FFmpegCPU()
	# Récup de nombre total de coeurs contenus dans la machine
	self.nbre_cpu = self.cl_nbre_cpu.nbre_cpu

    def setOutputFile(self, outputfile):
        self.outputfile = outputfile

    def setInputFile(self, inputfile):
        if type(inputfile) == tuple:
            self.inputfile = inputfile[0]
            self.audiofile = inputfile[1]
        else:
            self.inputfile = inputfile
            self.audiofile = None

    def setVideoLen(self, videoLen):
        """ Définie la durée de la vidéo """
        self.videoLen = videoLen

    def setProgress(self, val):
        """
            Définie le niveau de progression et envoie un signal progress(int)
        """
        self.progress = val
        self.emit(SIGNAL("progress(int)"), self.progress)

    def addLog(self, line):
        """
            Ajout une ligne de log dans self.log et envoie un signal log(string)
        """
        try:
            debug(u"Mencoder log : {0}".format(unicode(line, EkdConfig.coding)))
            self.log = "{0}\n{1}".format(self.log, unicode(line, EkdConfig.coding))
            self.emit(SIGNAL("log(QString)"), line)
        except:
            debug(u"Mencoder log : {0}".format(unicode(line, EkdConfig.coding)))
            self.log = "{0}\n{1}".format(self.log, line)
            self.emit(SIGNAL("log(QString)"), line)

    def run(self):
        """ Démarre le processus mencoder """
        if self.command == None :
            debug("FFmpeg: Commande mencoder non définie")
            raise Exception
        self.addLog(self.command.encode(EkdConfig.coding))
        try :
            self.process = EkdProcess( self.command.encode(EkdConfig.coding),
                                        stdinput = None)
            self.emit(SIGNAL("void started()"))
        except Exception, e:
            debug("Erreur dans le lancement de {0}\n({1})".format(self.command, e))
            raise
        self.retCode = None
        while self.retCode == None :
            self.progression()
            self.retCode = self.process.poll()
            self.usleep(10)
        self.end(self.retCode)

    def cancel(self):
        """ Cancel the process if it is currently running """
        try:
            self.process.terminate()
        except:
            pass

    def end(self, returnCode):
        """ Traite la fin du processus mencoder """
        if returnCode != 0:
            self.addLog(u"Une erreur est survenue "
                        "lors de l'encodage".encode(EkdConfig.coding))
            errors = ""
            self.process.stderr.flush()
            for line in self.process.stderr.readlines():
                errors += line
            self.addLog(QString(errors))
        self.emit(SIGNAL("finished(int)"), returnCode)

    def progression(self):
        """
        Recupération de la progression du process en cours
        """
        line = self.process.stdout.readline()
        if re.match('^Pos.*', line):
            val = re.findall("\d+%",line)[0]
            self.setProgress(int(val[:-1]))
        else:
            self.addLog(line)

    def setCommand(self, parametres, apercu = False):
        """
            Définie la ligne de commande de façon générique pour le traitement
            video
        """
        if parametres :
            debug (u"Mencoder : type de parametres : {0} len : {1} ".format(type(parametres), len(parametres)))
            for param in parametres :
                debug (u"Mencoder : parametres : {0}".format(param))
		
        if type(parametres) != list and type(parametres) != tuple and type(parametres) != dict:

            codecs_command = {
                'copie' : '-ovc copy -ovc lavc -lavcopts threads={0} -oac pcm'.format(self.nbre_cpu),
                'niveaudegris':'-ovc lavc -lavcopts gray:threads={0} -oac pcm'.format(self.nbre_cpu),
                'placesoustitres':'-ovc lavc -lavcopts threads={0} -vf expand=0:-50:0:0 -oac pcm'.format(self.nbre_cpu),
                'miroir':'-ovc lavc -lavcopts threads={0} -vf mirror -oac pcm'.format(self.nbre_cpu),
                'tournervideo':'-vf rotate=1 -oac pcm -ovc lavc -lavcopts threads={0}'.format(self.nbre_cpu),
                'extractionvideo': '-ovc lavc -lavcopts threads={0} -nosound'.format(self.nbre_cpu),
                'fusion_video': '-noidx -oac pcm -ovc lavc -lavcopts vcodec=mjpeg:vqmin=1:vpass=1:vbitrate=3000:threads={0}'.format(self.nbre_cpu),
                'fusion_audio_et_video_1': '-idx -audiofile \"{0}\" -oac lavc -lavcopts acodec=ac3 -stereo 0 -ovc copy -ovc lavc -lavcopts threads={1}'.format(self.audiofile, self.nbre_cpu),
                'fusion_audio_et_video_2': '-idx -audiofile \"{0}\" -oac pcm -stereo 0 -ovc lavc -lavcopts vcodec=mjpeg:vqmin=1:vpass=1:vbitrate=3000:threads={1}'.format(self.audiofile, self.nbre_cpu),
                }
        elif len(parametres) == 2 and type(parametres) != dict :
            # Cas de parametres standards
            codecs_command = {}
            codecs_command['luminositecontraste'] = '-ovc lavc -lavcopts threads={0} -vf eq={1}:{2} -oac pcm'.format(self.nbre_cpu, parametres[0], parametres[1])
            codecs_command['decoupervideo'] = '-ss {0} -endpos {1} -oac pcm -ovc copy -ovc lavc -lavcopts threads={2}'.format(parametres[0], parametres[1], self.nbre_cpu)
            codecs_command['couleursaturationhue'] ='-ovc lavc -lavcopts threads={0} -vf hue={1}:{2} -oac pcm'.format(self.nbre_cpu, parametres[0], parametres[1])
            codecs_command['decoupervideo_nosound'] = '-ss {0} -endpos {1} -nosound -ovc copy -ovc lavc -lavcopts threads={2}'.format(parametres[0], parametres[1], self.nbre_cpu)
            codecs_command['flouboitebloxblur'] = '-ovc lavc -lavcopts threads={0} -vf boxblur={1}:{2} -oac pcm'.format(self.nbre_cpu, parametres[0], parametres[1])
            codecs_command['changement_resolution'] = '-vf scale={0}:{1} -ovc lavc -lavcopts vcodec=mjpeg:vqmin=1:vpass=1:threads={2} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu)
	    codecs_command['avirawsanscompression'] = '-ovc raw -vf {0}format=i420{1} -ovc lavc -lavcopts threads={2} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu)
	elif len(parametres) == 4 and type(parametres) != dict :
            codecs_command = {}
	    codecs_command['jpeg_sans_perte'] = '-ovc lavc -lavcopts vcodec=ljpeg:vbitrate={0}:vpass=1:threads={1} {2}{3} {4} -oac pcm'.format(parametres[0], self.nbre_cpu, parametres[1], parametres[2], parametres[3])
            codecs_command['ffv1'] = '-ovc lavc -lavcopts vcodec=ffv1:vbitrate={0}:vpass=1:threads={1} {2}{3} {4} -oac pcm'.format(parametres[0], self.nbre_cpu, parametres[1], parametres[2], parametres[3])
	    codecs_command['codecxvid'] = '-ovc lavc -lavcopts threads={0} -ffourcc xvid -xvidencopts pass=1:bitrate={1} {2}{3} {4} -oac pcm'.format(self.nbre_cpu, parametres[0], parametres[1], parametres[2], parametres[3])
	    # Cas Luma et Chroma et découpage, on a besoin de 4 parametres
            codecs_command['bruit'] = '-ovc lavc -lavcopts threads={0} -vf noise={1}[{2}]:{3}[{4}] -oac pcm'.format(self.nbre_cpu, parametres[2], parametres[0], parametres[3], parametres[1])
            # Pas d'utilisation de plusieurs coeurs (threads nbre) ici pour le filtre vidéo découpage assisté car sinon plantage de Mencoder
	    codecs_command['decoupagelibre'] ='-ovc lavc -vf crop={0}:{1}:{2}:{3} -oac pcm'.format(parametres[0], parametres[1],parametres[2],parametres[3])
            # Pas d'utilisation de plusieurs coeurs (threads nbre) ici pour le filtre vidéo découpage libre car sinon plantage de Mencoder
	    codecs_command['decoupageassiste'] ='-ovc lavc -vf crop={0}:{1}:{2}:{3} -oac pcm'.format(parametres[0], parametres[1],parametres[2],parametres[3])
	elif len(parametres) == 5 and type(parametres) != dict :
            codecs_command = {}
	    codecs_command['codecdivx4'] = '-ovc lavc -lavcopts vcodec=mpeg4:vbitrate={0}{1}:vpass=1:threads={2} {3}{4} {5} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu, parametres[2], parametres[3], parametres[4])
	    codecs_command['codecmotionjpeg'] = '-ovc lavc -lavcopts vcodec=mjpeg:vbitrate={0}{1}:vpass=1:threads={2} {3}{4} {5} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu, parametres[2], parametres[3], parametres[4])
	    codecs_command['codecmpeg1'] = '-ovc lavc -lavcopts vcodec=mpeg1video:vbitrate={0}{1}:vpass=1:threads={2} {3}{4} {5} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu, parametres[2], parametres[3], parametres[4])
	    codecs_command['codecmpeg2'] = '-ovc lavc -lavcopts vcodec=mpeg2video:vbitrate={0}{1}:vpass=1:threads={2} {3}{4} {5} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu, parametres[2], parametres[3], parametres[4])
	    codecs_command['macromediaflashvideo'] = '-ovc lavc -ffourcc FLV1 -lavcopts vcodec=flv:vbitrate={0}{1}:threads={2} {3}{4} {5} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu, parametres[2], parametres[3], parametres[4])
	    codecs_command['codecwmv2'] = '-ovc lavc -lavcopts vcodec=wmv2:vbitrate={0}{1}:vpass=1:threads={2} {3}{4} {5} -oac pcm'.format(parametres[0], parametres[1], self.nbre_cpu, parametres[2], parametres[3], parametres[4])
	    # Pas d'utilisation de plusieurs coeurs (threads nbre) ici pour le h264mpeg4 extension h264 car sinon plantage de Mencoder
	    codecs_command['codech264mpeg4_ext_h264'] = '-nosound -of lavf -lavfopts format=h264 {0} -ovc x264 -x264encopts bitrate={1} {2} {3}'.format(parametres[2], parametres[0], parametres[3], parametres[4])
	    # Pas d'utilisation de plusieurs coeurs (threads nbre) ici pour le h263plus car sinon plantage de Mencoder
	    codecs_command['codech263p'] = '-ovc lavc -lavcopts vcodec=h263p:vbitrate={0}{1}:vpass=1 {2}{3} {4} -oac pcm'.format(parametres[0], parametres[1], parametres[2], parametres[3], parametres[4])
	else :
            codecs_command = {}
            info = ""
	    # Cas de Vidéo > Tags vidéo
            for key in parametres.keys():
                valeur = parametres[key.capitalize()] 
                if valeur == "":
                    ## On évite de planter la ligne de commande si un champ n'a pas été renseigné
                    valeur = " "
                if key == "Title" :
                    ## On traite le bug de Title dans la ligne de commande qui est en fait remplacée par name
                    key = "name"
                elif key == "Comments":
                    ## On traite le bug de Comments dans la ligne de commande qui est en fait remplacée par comment
                    key = "comment"
                info = '{0}:{1}="{2}"'.format(info, key.lower(), valeur)
            codecs_command['tag_video'] ='-ovc copy -oac copy -info {0}'.format(info.strip(":"))

        if apercu :
            apercu_cmd = "-ss {0} -frames 2".format(apercu)
        else:
            apercu_cmd = ""

        self.command = u"{0} {1} \"{2}\" {3} {4} -o \"{5}\"".format(self.location, self.dfltOptions, self.inputfile, codecs_command[self.codec], apercu_cmd, self.outputfile)
